@@ -27,7 +27,7 @@ add_task(async function() {
       let uri = Services.io.newURI("about:" + aboutType);
       let flags = am.getURIFlags(uri);
       if (!(flags & Ci.nsIAboutModule.HIDE_FROM_ABOUTABOUT) &&
-          networkURLs.indexOf(aboutType) == -1) {
+          !networkURLs.includes(aboutType)) {
         aboutURLs.push(aboutType);
       }
     } catch (e) {
