@@ -20,6 +20,8 @@ var {
   defineLazyGetter,
 } = ExtensionUtils;
 
+const READER_MODE_PREFIX = "about:reader";
+
 let tabTracker;
 let windowTracker;
 
@@ -205,6 +207,7 @@ class TabTracker extends TabTrackerBase {
     super();
 
     this._tabs = new WeakMap();
+    this._browsers = new WeakMap();
     this._tabIds = new Map();
     this._nextId = 1;
 
@@ -228,6 +231,8 @@ class TabTracker extends TabTrackerBase {
     windowTracker.addOpenListener(this._handleWindowOpen);
     windowTracker.addCloseListener(this._handleWindowClose);
 
+    Services.mm.addMessageListener("Reader:UpdateReaderButton", this);
+
     /* eslint-disable mozilla/balanced-listeners */
     this.on("tab-detached", this._handleTabDestroyed);
     this.on("tab-removed", this._handleTabDestroyed);
@@ -235,19 +240,38 @@ class TabTracker extends TabTrackerBase {
   }
 
   getId(nativeTab) {
-    if (this._tabs.has(nativeTab)) {
-      return this._tabs.get(nativeTab);
+    let id = this._tabs.get(nativeTab);
+    if (id) {
+      return id;
     }
 
     this.init();
 
-    let id = this._nextId++;
+    id = this._nextId++;
     this.setId(nativeTab, id);
     return id;
   }
 
+  getBrowserTabId(browser) {
+    let id = this._browsers.get(browser);
+    if (id) {
+      return id;
+    }
+
+    let tab = browser.ownerGlobal.gBrowser.getTabForBrowser(browser);
+    if (tab) {
+      id = this.getId(tab);
+      this._browsers.set(browser, id);
+      return id;
+    }
+    return -1;
+  }
+
   setId(nativeTab, id) {
     this._tabs.set(nativeTab, id);
+    if (nativeTab.linkedBrowser) {
+      this._browsers.set(nativeTab.linkedBrowser, id);
+    }
     this._tabIds.set(id, nativeTab);
   }
 
@@ -362,6 +386,21 @@ class TabTracker extends TabTrackerBase {
     }
   }
 
+  /**
+   * @param {Object} message
+   *        The message to handle.
+   * @private
+   */
+  receiveMessage(message) {
+    switch (message.name) {
+      case "Reader:UpdateReaderButton":
+        if (message.data && message.data.isArticle !== undefined) {
+          this.emit("tab-isarticle", message);
+        }
+        break;
+    }
+  }
+
   /**
    * A private method which is called whenever a new browser window is opened,
    * and dispatches the necessary events for it.
@@ -499,47 +538,31 @@ class TabTracker extends TabTrackerBase {
     let windowId = windowTracker.getId(nativeTab.ownerGlobal);
     let tabId = this.getId(nativeTab);
 
-    // When addons run in-process, `window.close()` is synchronous. Most other
-    // addon-invoked calls are asynchronous since they go through a proxy
-    // context via the message manager. This includes event registrations such
-    // as `tabs.onRemoved.addListener`.
-    //
-    // So, even if `window.close()` were to be called (in-process) after calling
-    // `tabs.onRemoved.addListener`, then the tab would be closed before the
-    // event listener is registered. To make sure that the event listener is
-    // notified, we dispatch `tabs.onRemoved` asynchronously.
-    Services.tm.dispatchToMainThread(() => {
-      this.emit("tab-removed", {nativeTab, tabId, windowId, isWindowClosing});
-    });
+    this.emit("tab-removed", {nativeTab, tabId, windowId, isWindowClosing});
   }
 
   getBrowserData(browser) {
-    if (browser.ownerGlobal.location.href === "about:addons") {
-      // When we're loaded into a <browser> inside about:addons, we need to go up
-      // one more level.
-      browser = browser.ownerGlobal.QueryInterface(Ci.nsIInterfaceRequestor)
-                       .getInterface(Ci.nsIDocShell)
-                       .chromeEventHandler;
-    }
-
-    let result = {
-      tabId: -1,
-      windowId: -1,
-    };
-
     let {gBrowser} = browser.ownerGlobal;
-    // Some non-browser windows have gBrowser but not
-    // getTabForBrowser!
-    if (gBrowser && gBrowser.getTabForBrowser) {
-      result.windowId = windowTracker.getId(browser.ownerGlobal);
+    // Some non-browser windows have gBrowser but not getTabForBrowser!
+    if (!gBrowser || !gBrowser.getTabForBrowser) {
+      if (browser.ownerDocument.documentURI === "about:addons") {
+        // When we're loaded into a <browser> inside about:addons, we need to go up
+        // one more level.
+        browser = browser.ownerDocument.docShell.chromeEventHandler;
 
-      let nativeTab = gBrowser.getTabForBrowser(browser);
-      if (nativeTab) {
-        result.tabId = this.getId(nativeTab);
+        ({gBrowser} = browser.ownerGlobal);
+      } else {
+        return {
+          tabId: -1,
+          windowId: -1,
+        };
       }
     }
 
-    return result;
+    return {
+      tabId: this.getBrowserTabId(browser),
+      windowId: windowTracker.getId(browser.ownerGlobal),
+    };
   }
 
   get activeTab() {
@@ -569,6 +592,10 @@ class Tab extends TabBase {
     return this.nativeTab.linkedBrowser;
   }
 
+  get discarded() {
+    return !this.nativeTab.linkedPanel;
+  }
+
   get frameLoader() {
     // If we don't have a frameLoader yet, just return a dummy with no width and
     // height.
@@ -644,6 +671,14 @@ class Tab extends TabBase {
     return windowTracker.getId(this.window);
   }
 
+  get isArticle() {
+    return this.nativeTab.linkedBrowser.isArticle;
+  }
+
+  get isInReaderMode() {
+    return this.url && this.url.startsWith(READER_MODE_PREFIX);
+  }
+
   /**
    * Converts session store data to an object compatible with the return value
    * of the convert() method, representing that data.
@@ -674,7 +709,10 @@ class Tab extends TabBase {
 
     if (extension.tabManager.hasTabPermission(tabData)) {
       let entries = tabData.state ? tabData.state.entries : tabData.entries;
-      let entry = entries[entries.length - 1];
+      let lastTabIndex = tabData.state ? tabData.state.index : tabData.index;
+      // We need to take lastTabIndex - 1 because the index in the tab data is
+      // 1-based rather than 0-based.
+      let entry = entries[lastTabIndex - 1];
       result.url = entry.url;
       result.title = entry.title;
       if (tabData.image) {
