@@ -6,6 +6,8 @@ ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 ChromeUtils.import("resource://gre/modules/Services.jsm");
 ChromeUtils.import("resource://gre/modules/AppConstants.jsm");
 
+ChromeUtils.defineModuleGetter(this, "HeadlessShell",
+                               "resource:///modules/HeadlessShell.jsm");
 ChromeUtils.defineModuleGetter(this, "LaterRun",
                                "resource:///modules/LaterRun.jsm");
 ChromeUtils.defineModuleGetter(this, "PrivateBrowsingUtils",
@@ -17,31 +19,6 @@ ChromeUtils.defineModuleGetter(this, "ShellService",
 XPCOMUtils.defineLazyServiceGetter(this, "WindowsUIUtils",
                                    "@mozilla.org/windows-ui-utils;1", "nsIWindowsUIUtils");
 
-const nsISupports            = Ci.nsISupports;
-
-const nsIBrowserDOMWindow    = Ci.nsIBrowserDOMWindow;
-const nsIBrowserHandler      = Ci.nsIBrowserHandler;
-const nsIBrowserHistory      = Ci.nsIBrowserHistory;
-const nsIChannel             = Ci.nsIChannel;
-const nsICommandLine         = Ci.nsICommandLine;
-const nsICommandLineHandler  = Ci.nsICommandLineHandler;
-const nsIContentHandler      = Ci.nsIContentHandler;
-const nsIDocShellTreeItem    = Ci.nsIDocShellTreeItem;
-const nsIDOMChromeWindow     = Ci.nsIDOMChromeWindow;
-const nsIDOMWindow           = Ci.nsIDOMWindow;
-const nsIFileURL             = Ci.nsIFileURL;
-const nsIInterfaceRequestor  = Ci.nsIInterfaceRequestor;
-const nsINetUtil             = Ci.nsINetUtil;
-const nsIPrefLocalizedString = Ci.nsIPrefLocalizedString;
-const nsISupportsString      = Ci.nsISupportsString;
-const nsIWebNavigation       = Ci.nsIWebNavigation;
-const nsIWebNavigationInfo   = Ci.nsIWebNavigationInfo;
-const nsICommandLineValidator = Ci.nsICommandLineValidator;
-
-const NS_BINDING_ABORTED = Cr.NS_BINDING_ABORTED;
-const NS_ERROR_WONT_HANDLE_CONTENT = 0x805d0001;
-const NS_ERROR_ABORT = Cr.NS_ERROR_ABORT;
-
 function shouldLoadURI(aURI) {
   if (aURI && !aURI.schemeIs("chrome"))
     return true;
@@ -55,7 +32,7 @@ function resolveURIInternal(aCmdLine, aArgument) {
   var uri = aCmdLine.resolveURI(aArgument);
   var uriFixup = Services.uriFixup;
 
-  if (!(uri instanceof nsIFileURL)) {
+  if (!(uri instanceof Ci.nsIFileURL)) {
     return uriFixup.createFixupURI(aArgument,
                                    uriFixup.FIXUP_FLAG_FIX_SCHEME_TYPOS);
   }
@@ -163,7 +140,7 @@ function getPostUpdateOverridePage(defaultOverridePage) {
 
   // The existence of silent or the non-existence of showURL in the actions both
   // mean that an override page should not be displayed.
-  if (actions.indexOf("silent") != -1 || actions.indexOf("showURL") == -1)
+  if (actions.includes("silent") || !actions.includes("showURL"))
     return "";
 
   return update.getProperty("openURL") || defaultOverridePage;
@@ -178,7 +155,7 @@ function openWindow(parent, url, target, features, args, noExternalArgs) {
     var argstring;
     if (args) {
       argstring = Cc["@mozilla.org/supports-string;1"]
-                    .createInstance(nsISupportsString);
+                    .createInstance(Ci.nsISupportsString);
       argstring.data = args;
     }
 
@@ -202,7 +179,7 @@ function openWindow(parent, url, target, features, args, noExternalArgs) {
                        .createInstance(Ci.nsIMutableArray);
     stringArgs.forEach(function(uri) {
       var sstring = Cc["@mozilla.org/supports-string;1"]
-                      .createInstance(nsISupportsString);
+                      .createInstance(Ci.nsISupportsString);
       sstring.data = uri;
       uriArray.appendElement(sstring);
     });
@@ -243,15 +220,11 @@ function openPreferences(extraArgs) {
                          args);
 }
 
-function logSystemBasedSearch(engine) {
+function doSearch(searchTerm, cmdLine) {
+  var engine = Services.search.defaultEngine;
   var countId = (engine.identifier || ("other-" + engine.name)) + ".system";
   var count = Services.telemetry.getKeyedHistogramById("SEARCH_COUNTS");
   count.add(countId);
-}
-
-function doSearch(searchTerm, cmdLine) {
-  var engine = Services.search.defaultEngine;
-  logSystemBasedSearch(engine);
 
   var submission = engine.getSubmission(searchTerm, null, "system");
 
@@ -306,10 +279,10 @@ nsBrowserContentHandler.prototype = {
   },
 
   /* nsISupports */
-  QueryInterface: XPCOMUtils.generateQI([nsICommandLineHandler,
-                                         nsIBrowserHandler,
-                                         nsIContentHandler,
-                                         nsICommandLineValidator]),
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsICommandLineHandler,
+                                         Ci.nsIBrowserHandler,
+                                         Ci.nsIContentHandler,
+                                         Ci.nsICommandLineValidator]),
 
   /* nsICommandLineHandler */
   handle: function bch_handle(cmdLine) {
@@ -329,7 +302,7 @@ nsBrowserContentHandler.prototype = {
     // scripts or applications handle the situation as if Firefox was not
     // already running.
     if (cmdLine.handleFlag("remote", true)) {
-      throw NS_ERROR_ABORT;
+      throw Cr.NS_ERROR_ABORT;
     }
 
     var uriparam;
@@ -350,7 +323,8 @@ nsBrowserContentHandler.prototype = {
     try {
       while ((uriparam = cmdLine.handleFlagWithParam("new-tab", false))) {
         let uri = resolveURIInternal(cmdLine, uriparam);
-        handURIToExistingBrowser(uri, nsIBrowserDOMWindow.OPEN_NEWTAB, cmdLine, false,
+        handURIToExistingBrowser(uri, Ci.nsIBrowserDOMWindow.OPEN_NEWTAB,
+                                 cmdLine, false,
                                  Services.scriptSecurityManager.getSystemPrincipal());
         cmdLine.preventDefault = true;
       }
@@ -399,7 +373,8 @@ nsBrowserContentHandler.prototype = {
       var privateWindowParam = cmdLine.handleFlagWithParam("private-window", false);
       if (privateWindowParam) {
         let resolvedURI = resolveURIInternal(cmdLine, privateWindowParam);
-        handURIToExistingBrowser(resolvedURI, nsIBrowserDOMWindow.OPEN_NEWTAB, cmdLine, true,
+        handURIToExistingBrowser(resolvedURI, Ci.nsIBrowserDOMWindow.OPEN_NEWTAB,
+                                 cmdLine, true,
                                  Services.scriptSecurityManager.getSystemPrincipal());
         cmdLine.preventDefault = true;
       }
@@ -464,6 +439,10 @@ nsBrowserContentHandler.prototype = {
     } else {
       info += "  --preferences      Open Preferences dialog.\n";
     }
+    if (AppConstants.platform == "win" || AppConstants.MOZ_WIDGET_GTK) {
+      info += "  --screenshot [<path>] Save screenshot to <path> or in working directory.\n";
+      info += "  --window-size width[,height] Width and optionally height of screenshot.\n";
+    }
     info += "  --search <term>    Search <term> with your default search engine.\n";
     return info;
   },
@@ -561,11 +540,11 @@ nsBrowserContentHandler.prototype = {
 
   get startPage() {
     var uri = Services.prefs.getComplexValue("browser.startup.homepage",
-                                             nsIPrefLocalizedString).data;
+                                             Ci.nsIPrefLocalizedString).data;
     if (!uri) {
       Services.prefs.clearUserPref("browser.startup.homepage");
       uri = Services.prefs.getComplexValue("browser.startup.homepage",
-                                           nsIPrefLocalizedString).data;
+                                           Ci.nsIPrefLocalizedString).data;
     }
     return uri;
   },
@@ -605,9 +584,11 @@ nsBrowserContentHandler.prototype = {
   /* nsIContentHandler */
 
   handleContent: function bch_handleContent(contentType, context, request) {
+    const NS_ERROR_WONT_HANDLE_CONTENT = 0x805d0001;
+
     try {
       var webNavInfo = Cc["@mozilla.org/webnavigation-info;1"]
-                         .getService(nsIWebNavigationInfo);
+                         .getService(Ci.nsIWebNavigationInfo);
       if (!webNavInfo.isTypeSupported(contentType, null)) {
         throw NS_ERROR_WONT_HANDLE_CONTENT;
       }
@@ -615,10 +596,10 @@ nsBrowserContentHandler.prototype = {
       throw NS_ERROR_WONT_HANDLE_CONTENT;
     }
 
-    request.QueryInterface(nsIChannel);
-    handURIToExistingBrowser(request.URI, nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW, null, false,
-                             request.loadInfo.triggeringPrincipal);
-    request.cancel(NS_BINDING_ABORTED);
+    request.QueryInterface(Ci.nsIChannel);
+    handURIToExistingBrowser(request.URI, Ci.nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW,
+                             null, false, request.loadInfo.triggeringPrincipal);
+    request.cancel(Cr.NS_BINDING_ABORTED);
   },
 
   /* nsICommandLineValidator */
@@ -628,10 +609,10 @@ nsBrowserContentHandler.prototype = {
     var osintFlagIdx = cmdLine.findFlag("osint", false);
     var urlFlagIdx = cmdLine.findFlag("url", false);
     if (urlFlagIdx > -1 && (osintFlagIdx > -1 ||
-        cmdLine.state == nsICommandLine.STATE_REMOTE_EXPLICIT)) {
+        cmdLine.state == Ci.nsICommandLine.STATE_REMOTE_EXPLICIT)) {
       var urlParam = cmdLine.getArgument(urlFlagIdx + 1);
       if (cmdLine.length != urlFlagIdx + 2 || /firefoxurl:/.test(urlParam))
-        throw NS_ERROR_ABORT;
+        throw Cr.NS_ERROR_ABORT;
       var isDefault = false;
       try {
         var url = Services.urlFormatter.formatURLPref("app.support.baseURL") +
@@ -643,7 +624,7 @@ nsBrowserContentHandler.prototype = {
       if (isDefault) {
         // Firefox is already the default HTTP handler.
         // We don't have to show the instruction page.
-        throw NS_ERROR_ABORT;
+        throw Cr.NS_ERROR_ABORT;
       }
       cmdLine.handleFlag("osint", false);
     }
@@ -669,14 +650,14 @@ function handURIToExistingBrowser(uri, location, cmdLine, forcePrivate, triggeri
     return;
   }
 
-  var navNav = navWin.QueryInterface(nsIInterfaceRequestor)
-                     .getInterface(nsIWebNavigation);
-  var rootItem = navNav.QueryInterface(nsIDocShellTreeItem).rootTreeItem;
-  var rootWin = rootItem.QueryInterface(nsIInterfaceRequestor)
-                        .getInterface(nsIDOMWindow);
-  var bwin = rootWin.QueryInterface(nsIDOMChromeWindow).browserDOMWindow;
+  var navNav = navWin.QueryInterface(Ci.nsIInterfaceRequestor)
+                     .getInterface(Ci.nsIWebNavigation);
+  var rootItem = navNav.QueryInterface(Ci.nsIDocShellTreeItem).rootTreeItem;
+  var rootWin = rootItem.QueryInterface(Ci.nsIInterfaceRequestor)
+                        .getInterface(Ci.nsIDOMWindow);
+  var bwin = rootWin.QueryInterface(Ci.nsIDOMChromeWindow).browserDOMWindow;
   bwin.openURI(uri, null, location,
-               nsIBrowserDOMWindow.OPEN_EXTERNAL, triggeringPrincipal);
+               Ci.nsIBrowserDOMWindow.OPEN_EXTERNAL, triggeringPrincipal);
 }
 
 function nsDefaultCommandLineHandler() {
@@ -687,8 +668,8 @@ nsDefaultCommandLineHandler.prototype = {
 
   /* nsISupports */
   QueryInterface: function dch_QI(iid) {
-    if (!iid.equals(nsISupports) &&
-        !iid.equals(nsICommandLineHandler))
+    if (!iid.equals(Ci.nsISupports) &&
+        !iid.equals(Ci.nsICommandLineHandler))
       throw Cr.NS_ERROR_NO_INTERFACE;
 
     return this;
@@ -729,6 +710,11 @@ nsDefaultCommandLineHandler.prototype = {
       Cu.reportError(e);
     }
 
+    if (cmdLine.findFlag("screenshot", true) != -1) {
+      HeadlessShell.handleCmdLineArgs(cmdLine, urilist.filter(shouldLoadURI).map(u => u.spec));
+      return;
+    }
+
     for (let i = 0; i < cmdLine.length; ++i) {
       var curarg = cmdLine.getArgument(i);
       if (curarg.match(/^-/)) {
@@ -746,12 +732,13 @@ nsDefaultCommandLineHandler.prototype = {
     }
 
     if (urilist.length) {
-      if (cmdLine.state != nsICommandLine.STATE_INITIAL_LAUNCH &&
+      if (cmdLine.state != Ci.nsICommandLine.STATE_INITIAL_LAUNCH &&
           urilist.length == 1) {
         // Try to find an existing window and load our URI into the
         // current tab, new tab, or new window as prefs determine.
         try {
-          handURIToExistingBrowser(urilist[0], nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW, cmdLine, false,
+          handURIToExistingBrowser(urilist[0], Ci.nsIBrowserDOMWindow.OPEN_DEFAULTWINDOW,
+                                   cmdLine, false,
                                    Services.scriptSecurityManager.getSystemPrincipal());
           return;
         } catch (e) {
@@ -767,7 +754,7 @@ nsDefaultCommandLineHandler.prototype = {
 
     } else if (!cmdLine.preventDefault) {
       if (AppConstants.isPlatformAndVersionAtLeast("win", "10") &&
-          cmdLine.state != nsICommandLine.STATE_INITIAL_LAUNCH &&
+          cmdLine.state != Ci.nsICommandLine.STATE_INITIAL_LAUNCH &&
           WindowsUIUtils.inTabletMode) {
         // In windows 10 tablet mode, do not create a new window, but reuse the existing one.
         let win = RecentWindow.getMostRecentBrowserWindow();
