@@ -5,15 +5,44 @@
 
 const CC = Components.Constructor;
 
+ChromeUtils.defineModuleGetter(this, "AppConstants",
+  "resource://gre/modules/AppConstants.jsm");
+
 const TEST_PATH = "http://example.net/browser/browser/" +
                   "components/resistfingerprinting/test/browser/"
 
 var spoofedUserAgent;
 
-const SPOOFED_APPNAME        = "Netscape";
-const SPOOFED_APPVERSION     = "5.0 (Windows)";
-const SPOOFED_PLATFORM       = "Win64";
-const SPOOFED_OSCPU          = "Windows NT 6.1; Win64; x64";
+const SPOOFED_APPNAME = "Netscape";
+
+const SPOOFED_APPVERSION = {
+  linux: "5.0 (X11)",
+  win: "5.0 (Windows)",
+  macosx: "5.0 (Macintosh)",
+  android: "5.0 (Android 6.0)",
+  other: "5.0 (X11)",
+};
+const SPOOFED_PLATFORM = {
+  linux: "Linux x86_64",
+  win: "Win64",
+  macosx: "MacIntel",
+  android: "Linux armv7l",
+  other: "Linux x86_64",
+};
+const SPOOFED_OSCPU = {
+  linux: "Linux x86_64",
+  win: "Windows NT 6.1; Win64; x64",
+  macosx: "Intel Mac OS X 10.13",
+  android: "Linux armv7l",
+  other: "Linux x86_64",
+};
+const SPOOFED_UA_OS = {
+  linux: "X11; Linux x86_64",
+  win: "Windows NT 6.1; Win64; x64",
+  macosx: "Macintosh; Intel Mac OS X 10.13",
+  android: "Android 6.0; Mobile",
+  other: "X11; Linux x86_64",
+};
 const SPOOFED_BUILDID        = "20100101";
 const SPOOFED_HW_CONCURRENCY = 2;
 
@@ -35,12 +64,12 @@ async function testNavigator() {
   result = JSON.parse(result);
 
   is(result.appName, SPOOFED_APPNAME, "Navigator.appName is correctly spoofed.");
-  is(result.appVersion, SPOOFED_APPVERSION, "Navigator.appVersion is correctly spoofed.");
-  is(result.platform, SPOOFED_PLATFORM, "Navigator.platform is correctly spoofed.");
+  is(result.appVersion, SPOOFED_APPVERSION[AppConstants.platform], "Navigator.appVersion is correctly spoofed.");
+  is(result.platform, SPOOFED_PLATFORM[AppConstants.platform], "Navigator.platform is correctly spoofed.");
   is(result.userAgent, spoofedUserAgent, "Navigator.userAgent is correctly spoofed.");
   is(result.mimeTypesLength, 0, "Navigator.mimeTypes has a length of 0.");
   is(result.pluginsLength, 0, "Navigator.plugins has a length of 0.");
-  is(result.oscpu, SPOOFED_OSCPU, "Navigator.oscpu is correctly spoofed.");
+  is(result.oscpu, SPOOFED_OSCPU[AppConstants.platform], "Navigator.oscpu is correctly spoofed.");
   is(result.buildID, SPOOFED_BUILDID, "Navigator.buildID is correctly spoofed.");
   is(result.hardwareConcurrency, SPOOFED_HW_CONCURRENCY, "Navigator.hardwareConcurrency is correctly spoofed.");
 
@@ -73,8 +102,8 @@ async function testWorkerNavigator() {
   result = JSON.parse(result);
 
   is(result.appName, SPOOFED_APPNAME, "Navigator.appName is correctly spoofed.");
-  is(result.appVersion, SPOOFED_APPVERSION, "Navigator.appVersion is correctly spoofed.");
-  is(result.platform, SPOOFED_PLATFORM, "Navigator.platform is correctly spoofed.");
+  is(result.appVersion, SPOOFED_APPVERSION[AppConstants.platform], "Navigator.appVersion is correctly spoofed.");
+  is(result.platform, SPOOFED_PLATFORM[AppConstants.platform], "Navigator.platform is correctly spoofed.");
   is(result.userAgent, spoofedUserAgent, "Navigator.userAgent is correctly spoofed.");
   is(result.hardwareConcurrency, SPOOFED_HW_CONCURRENCY, "Navigator.hardwareConcurrency is correctly spoofed.");
 
@@ -91,7 +120,7 @@ add_task(async function setup() {
 
   let appVersion = parseInt(Services.appinfo.version);
   let spoofedVersion = appVersion - ((appVersion - 3) % 7);
-  spoofedUserAgent = `Mozilla/5.0 (${SPOOFED_OSCPU}; rv:${spoofedVersion}.0) Gecko/20100101 Firefox/${spoofedVersion}.0`;
+  spoofedUserAgent = `Mozilla/5.0 (${SPOOFED_UA_OS[AppConstants.platform]}; rv:${spoofedVersion}.0) Gecko/20100101 Firefox/${spoofedVersion}.0`;
 });
 
 add_task(async function runNavigatorTest() {
