@@ -6,21 +6,21 @@
  *  1. pause in the main thread
  *  2. pause in the iframe
  */
-add_task(function*() {
-  const dbg = yield initDebugger("doc-iframes.html");
+add_task(async function() {
+  const dbg = await initDebugger("doc-iframes.html");
 
   // test pausing in the main thread
-  yield reload(dbg);
-  yield waitForPaused(dbg);
+  await reload(dbg);
+  await waitForPaused(dbg);
   assertPausedLocation(dbg);
 
   // test pausing in the iframe
-  yield resume(dbg);
-  yield waitForPaused(dbg);
+  await resume(dbg);
+  await waitForPaused(dbg);
   assertPausedLocation(dbg);
 
   // test pausing in the iframe
-  yield resume(dbg);
-  yield waitForPaused(dbg);
+  await resume(dbg);
+  await waitForPaused(dbg);
   assertPausedLocation(dbg);
 });
