@@ -4,11 +4,11 @@
 
 "use strict";
 
-const {NetUtil} = Cu.import("resource://gre/modules/NetUtil.jsm", {});
-const {FileUtils} = Cu.import("resource://gre/modules/FileUtils.jsm", {});
-const {console} = Cu.import("resource://gre/modules/Console.jsm", {});
-const {ScratchpadManager} = Cu.import("resource://devtools/client/scratchpad/scratchpad-manager.jsm", {});
-const {require} = Cu.import("resource://devtools/shared/Loader.jsm", {});
+const {NetUtil} = ChromeUtils.import("resource://gre/modules/NetUtil.jsm", {});
+const {FileUtils} = ChromeUtils.import("resource://gre/modules/FileUtils.jsm", {});
+const {console} = ChromeUtils.import("resource://gre/modules/Console.jsm", {});
+const {ScratchpadManager} = ChromeUtils.import("resource://devtools/client/scratchpad/scratchpad-manager.jsm", {});
+const {require} = ChromeUtils.import("resource://devtools/shared/Loader.jsm", {});
 const Services = require("Services");
 const DevToolsUtils = require("devtools/shared/DevToolsUtils");
 const flags = require("devtools/shared/flags");
