@@ -6,7 +6,6 @@
 
 const { Ci } = require("chrome");
 const promise = require("promise");
-const defer = require("devtools/shared/defer");
 const Services = require("Services");
 const { TargetFactory } = require("devtools/client/framework/target");
 const Telemetry = require("devtools/client/shared/telemetry");
@@ -309,25 +308,23 @@ DeveloperToolbar.prototype.focus = function () {
   if (this.visible) {
     // If the toolbar was just inserted, the <textbox> may still have
     // its binding in process of being applied and not be focusable yet
-    let waitForBinding = defer();
-
-    let checkBinding = () => {
-      // Bail out if the toolbar has been destroyed in the meantime
-      if (!this._input) {
-        waitForBinding.reject();
-        return;
-      }
-      // mInputField is a xbl field of <xul:textbox>
-      if (typeof this._input.mInputField != "undefined") {
-        this._input.focus();
-        waitForBinding.resolve();
-      } else {
-        this._input.ownerDocument.defaultView.setTimeout(checkBinding, 50);
-      }
-    };
-    checkBinding();
-
-    return waitForBinding.promise;
+    return new Promise((resolve, reject) => {
+      let checkBinding = () => {
+        // Bail out if the toolbar has been destroyed in the meantime
+        if (!this._input) {
+          reject();
+          return;
+        }
+        // mInputField is a xbl field of <xul:textbox>
+        if (typeof this._input.mInputField != "undefined") {
+          this._input.focus();
+          resolve();
+        } else {
+          this._input.ownerDocument.defaultView.setTimeout(checkBinding, 50);
+        }
+      };
+      checkBinding();
+    });
   }
 
   return this.show(true);
@@ -817,77 +814,76 @@ OutputPanel.create = function (devtoolbar) {
  * @private See OutputPanel.create
  */
 OutputPanel.prototype._init = function (devtoolbar) {
-  this._devtoolbar = devtoolbar;
-  this._input = this._devtoolbar._input;
-  this._toolbar = this._devtoolbar._doc.getElementById("developer-toolbar");
+  return new Promise((resolve, reject) => {
+    this._devtoolbar = devtoolbar;
+    this._input = this._devtoolbar._input;
+    this._toolbar = this._devtoolbar._doc.getElementById("developer-toolbar");
 
-  /*
-  <tooltip|panel id="gcli-output"
-         noautofocus="true"
-         noautohide="true"
-         class="gcli-panel">
-    <html:iframe xmlns:html="http://www.w3.org/1999/xhtml"
-                 id="gcli-output-frame"
-                 src="chrome://devtools/content/commandline/commandlineoutput.xhtml"
-                 sandbox="allow-same-origin"/>
-  </tooltip|panel>
-  */
+    /*
+    <tooltip|panel id="gcli-output"
+           noautofocus="true"
+           noautohide="true"
+           class="gcli-panel">
+      <html:iframe xmlns:html="http://www.w3.org/1999/xhtml"
+                   id="gcli-output-frame"
+                   src="chrome://devtools/content/commandline/commandlineoutput.xhtml"
+                   sandbox="allow-same-origin"/>
+    </tooltip|panel>
+    */
 
-  // TODO: Switch back from tooltip to panel when metacity focus issue is fixed:
-  // https://bugzilla.mozilla.org/show_bug.cgi?id=780102
-  this._panel = this._devtoolbar._doc.createElement(isLinux ? "tooltip" : "panel");
+    // TODO: Switch back from tooltip to panel when metacity focus issue is fixed:
+    // https://bugzilla.mozilla.org/show_bug.cgi?id=780102
+    this._panel = this._devtoolbar._doc.createElement(isLinux ? "tooltip" : "panel");
 
-  this._panel.id = "gcli-output";
-  this._panel.classList.add("gcli-panel");
+    this._panel.id = "gcli-output";
+    this._panel.classList.add("gcli-panel");
 
-  if (isLinux) {
-    this.canHide = false;
-    this._onpopuphiding = this._onpopuphiding.bind(this);
-    this._panel.addEventListener("popuphiding", this._onpopuphiding, true);
-  } else {
-    this._panel.setAttribute("noautofocus", "true");
-    this._panel.setAttribute("noautohide", "true");
+    if (isLinux) {
+      this.canHide = false;
+      this._onpopuphiding = this._onpopuphiding.bind(this);
+      this._panel.addEventListener("popuphiding", this._onpopuphiding, true);
+    } else {
+      this._panel.setAttribute("noautofocus", "true");
+      this._panel.setAttribute("noautohide", "true");
 
-    // Bug 692348: On Windows and OSX if a panel has no content and no height
-    // openPopup fails to display it. Setting the height to 1px alows the panel
-    // to be displayed before has content or a real height i.e. the first time
-    // it is displayed.
-    this._panel.setAttribute("height", "1px");
-  }
+      // Bug 692348: On Windows and OSX if a panel has no content and no height
+      // openPopup fails to display it. Setting the height to 1px alows the panel
+      // to be displayed before has content or a real height i.e. the first time
+      // it is displayed.
+      this._panel.setAttribute("height", "1px");
+    }
 
-  this._toolbar.parentElement.insertBefore(this._panel, this._toolbar);
+    this._toolbar.parentElement.insertBefore(this._panel, this._toolbar);
 
-  this._frame = this._devtoolbar._doc.createElementNS(NS_XHTML, "iframe");
-  this._frame.id = "gcli-output-frame";
-  this._frame.setAttribute("src", "chrome://devtools/content/commandline/commandlineoutput.xhtml");
-  this._frame.setAttribute("sandbox", "allow-same-origin");
-  this._panel.appendChild(this._frame);
+    this._frame = this._devtoolbar._doc.createElementNS(NS_XHTML, "iframe");
+    this._frame.id = "gcli-output-frame";
+    this._frame.setAttribute("src", "chrome://devtools/content/commandline/commandlineoutput.xhtml");
+    this._frame.setAttribute("sandbox", "allow-same-origin");
+    this._panel.appendChild(this._frame);
 
-  this.displayedOutput = undefined;
+    this.displayedOutput = undefined;
 
-  this._update = this._update.bind(this);
+    this._update = this._update.bind(this);
 
-  // Wire up the element from the iframe, and resolve the promise
-  let deferred = defer();
-  let onload = () => {
-    this._frame.removeEventListener("load", onload, true);
+    // Wire up the element from the iframe, and resolve the promise
+    let onload = () => {
+      this._frame.removeEventListener("load", onload, true);
 
-    this.document = this._frame.contentDocument;
-    this._copyTheme();
+      this.document = this._frame.contentDocument;
+      this._copyTheme();
 
-    this._div = this.document.getElementById("gcli-output-root");
-    this._div.classList.add("gcli-row-out");
-    this._div.setAttribute("aria-live", "assertive");
+      this._div = this.document.getElementById("gcli-output-root");
+      this._div.classList.add("gcli-row-out");
+      this._div.setAttribute("aria-live", "assertive");
 
-    let styles = this._toolbar.ownerDocument.defaultView
-                    .getComputedStyle(this._toolbar);
-    this._div.setAttribute("dir", styles.direction);
+      let styles = this._toolbar.ownerDocument.defaultView
+                      .getComputedStyle(this._toolbar);
+      this._div.setAttribute("dir", styles.direction);
 
-    deferred.resolve(this);
-  };
-  this._frame.addEventListener("load", onload, true);
-
-  return deferred.promise;
+      resolve(this);
+    };
+    this._frame.addEventListener("load", onload, true);
+  });
 };
 
 /* Copy the current devtools theme attribute into the iframe,
@@ -1136,78 +1132,76 @@ TooltipPanel.create = function (devtoolbar) {
  * @private See TooltipPanel.create
  */
 TooltipPanel.prototype._init = function (devtoolbar) {
-  let deferred = defer();
+  return new Promise((resolve, reject) => {
+    this._devtoolbar = devtoolbar;
+    this._input = devtoolbar._doc.querySelector(".gclitoolbar-input-node");
+    this._toolbar = devtoolbar._doc.querySelector("#developer-toolbar");
+    this._dimensions = { start: 0, end: 0 };
 
-  this._devtoolbar = devtoolbar;
-  this._input = devtoolbar._doc.querySelector(".gclitoolbar-input-node");
-  this._toolbar = devtoolbar._doc.querySelector("#developer-toolbar");
-  this._dimensions = { start: 0, end: 0 };
+    /*
+    <tooltip|panel id="gcli-tooltip"
+           type="arrow"
+           noautofocus="true"
+           noautohide="true"
+           class="gcli-panel">
+      <html:iframe xmlns:html="http://www.w3.org/1999/xhtml"
+                   id="gcli-tooltip-frame"
+                   src="chrome://devtools/content/commandline/commandlinetooltip.xhtml"
+                   flex="1"
+                   sandbox="allow-same-origin"/>
+    </tooltip|panel>
+    */
 
-  /*
-  <tooltip|panel id="gcli-tooltip"
-         type="arrow"
-         noautofocus="true"
-         noautohide="true"
-         class="gcli-panel">
-    <html:iframe xmlns:html="http://www.w3.org/1999/xhtml"
-                 id="gcli-tooltip-frame"
-                 src="chrome://devtools/content/commandline/commandlinetooltip.xhtml"
-                 flex="1"
-                 sandbox="allow-same-origin"/>
-  </tooltip|panel>
-  */
+    // TODO: Switch back from tooltip to panel when metacity focus issue is fixed:
+    // https://bugzilla.mozilla.org/show_bug.cgi?id=780102
+    this._panel = devtoolbar._doc.createElement(isLinux ? "tooltip" : "panel");
 
-  // TODO: Switch back from tooltip to panel when metacity focus issue is fixed:
-  // https://bugzilla.mozilla.org/show_bug.cgi?id=780102
-  this._panel = devtoolbar._doc.createElement(isLinux ? "tooltip" : "panel");
+    this._panel.id = "gcli-tooltip";
+    this._panel.classList.add("gcli-panel");
 
-  this._panel.id = "gcli-tooltip";
-  this._panel.classList.add("gcli-panel");
+    if (isLinux) {
+      this.canHide = false;
+      this._onpopuphiding = this._onpopuphiding.bind(this);
+      this._panel.addEventListener("popuphiding", this._onpopuphiding, true);
+    } else {
+      this._panel.setAttribute("noautofocus", "true");
+      this._panel.setAttribute("noautohide", "true");
 
-  if (isLinux) {
-    this.canHide = false;
-    this._onpopuphiding = this._onpopuphiding.bind(this);
-    this._panel.addEventListener("popuphiding", this._onpopuphiding, true);
-  } else {
-    this._panel.setAttribute("noautofocus", "true");
-    this._panel.setAttribute("noautohide", "true");
+      // Bug 692348: On Windows and OSX if a panel has no content and no height
+      // openPopup fails to display it. Setting the height to 1px alows the panel
+      // to be displayed before has content or a real height i.e. the first time
+      // it is displayed.
+      this._panel.setAttribute("height", "1px");
+    }
 
-    // Bug 692348: On Windows and OSX if a panel has no content and no height
-    // openPopup fails to display it. Setting the height to 1px alows the panel
-    // to be displayed before has content or a real height i.e. the first time
-    // it is displayed.
-    this._panel.setAttribute("height", "1px");
-  }
+    this._toolbar.parentElement.insertBefore(this._panel, this._toolbar);
 
-  this._toolbar.parentElement.insertBefore(this._panel, this._toolbar);
+    this._frame = devtoolbar._doc.createElementNS(NS_XHTML, "iframe");
+    this._frame.id = "gcli-tooltip-frame";
+    this._frame.setAttribute("src", "chrome://devtools/content/commandline/commandlinetooltip.xhtml");
+    this._frame.setAttribute("flex", "1");
+    this._frame.setAttribute("sandbox", "allow-same-origin");
+    this._panel.appendChild(this._frame);
 
-  this._frame = devtoolbar._doc.createElementNS(NS_XHTML, "iframe");
-  this._frame.id = "gcli-tooltip-frame";
-  this._frame.setAttribute("src", "chrome://devtools/content/commandline/commandlinetooltip.xhtml");
-  this._frame.setAttribute("flex", "1");
-  this._frame.setAttribute("sandbox", "allow-same-origin");
-  this._panel.appendChild(this._frame);
+    /**
+     * Wire up the element from the iframe, and resolve the promise.
+     */
+    let onload = () => {
+      this._frame.removeEventListener("load", onload, true);
 
-  /**
-   * Wire up the element from the iframe, and resolve the promise.
-   */
-  let onload = () => {
-    this._frame.removeEventListener("load", onload, true);
+      this.document = this._frame.contentDocument;
+      this._copyTheme();
+      this.hintElement = this.document.getElementById("gcli-tooltip-root");
+      this._connector = this.document.getElementById("gcli-tooltip-connector");
 
-    this.document = this._frame.contentDocument;
-    this._copyTheme();
-    this.hintElement = this.document.getElementById("gcli-tooltip-root");
-    this._connector = this.document.getElementById("gcli-tooltip-connector");
+      let styles = this._toolbar.ownerDocument.defaultView
+                      .getComputedStyle(this._toolbar);
+      this.hintElement.setAttribute("dir", styles.direction);
 
-    let styles = this._toolbar.ownerDocument.defaultView
-                    .getComputedStyle(this._toolbar);
-    this.hintElement.setAttribute("dir", styles.direction);
-
-    deferred.resolve(this);
-  };
-  this._frame.addEventListener("load", onload, true);
-
-  return deferred.promise;
+      resolve(this);
+    };
+    this._frame.addEventListener("load", onload, true);
+  });
 };
 
 /* Copy the current devtools theme attribute into the iframe,
