@@ -49,18 +49,6 @@ pref("devtools.webconsole.autoMultiline", true);
 const NewConsoleOutputWrapper = require("../new-console-output/new-console-output-wrapper");
 const NewWebConsoleFrame = require("../new-webconsole").NewWebConsoleFrame;
 
-// Replicate the DOM that the root component lives within
-const el = document.createElement("div");
-el.style.flex = "1";
-el.innerHTML = `
-  <div id="app-wrapper" class="theme-body">
-    <div id="output-container" role="document" aria-live="polite" />
-  </div>
-`;
-document.querySelector("#mount").appendChild(el);
-
-document.documentElement.classList.add("theme-light");
-
 // Copied from netmonitor/index.js:
 window.addEventListener("DOMContentLoaded", () => {
   for (let link of document.head.querySelectorAll("link")) {
@@ -83,6 +71,13 @@ function onConnect(connection) {
     return;
   }
 
+  // Replicate the DOM that the root component lives within
+  document.querySelector("#mount").innerHTML = `
+    <div id="app-wrapper" class="theme-body">
+      <div id="output-container" role="document" aria-live="polite" />
+    </div>
+  `;
+
   // Stub out properties that are received from hudservice
   const owner = {
     iframeWindow: window,
@@ -107,4 +102,5 @@ window.evaluateJS = function (input) {
   }, {});
 };
 
-bootstrap(React, ReactDOM, el).then(onConnect);
+document.documentElement.classList.add("theme-light");
+bootstrap(React, ReactDOM).then(onConnect);
