@@ -26,17 +26,23 @@ let webpackConfig = {
         loader: "file-loader?name=[path][name].[ext]",
       },
       {
-        /*
-         * The version of webpack used in the launchpad seems to have trouble
-         * with the require("raw!${file}") that we use for the properties
-         * file in l10.js.
-         * This loader goes through the whole code and remove the "raw!" prefix
-         * so the raw-loader declared in devtools-launchpad config can load
-         * those files.
-         */
-        test: /\.js/,
-        loader: "rewrite-raw",
-      },
+        test: /\.js$/,
+        loaders: [
+          /*
+          * The version of webpack used in the launchpad seems to have trouble
+          * with the require("raw!${file}") that we use for the properties
+          * file in l10.js.
+          * This loader goes through the whole code and remove the "raw!" prefix
+          * so the raw-loader declared in devtools-launchpad config can load
+          * those files.
+          */
+          "rewrite-raw",
+          // Replace all references to this.browserRequire() by require()
+          "rewrite-browser-require",
+          // Replace all references to loader.lazyRequire() by require()
+          "rewrite-lazy-require",
+        ],
+      }
     ]
   },
 
@@ -85,7 +91,7 @@ webpackConfig.resolve = {
     "devtools/client/locales": path.join(__dirname, "../../client/locales/en-US"),
     "toolkit/locales": path.join(__dirname, "../../../toolkit/locales/en-US"),
     "devtools/shared/locales": path.join(__dirname, "../../shared/locales/en-US"),
-    "devtools/shim/locales": path.join(__dirname, "../../shared/locales/en-US"),
+    "devtools/startup/locales": path.join(__dirname, "../../shared/locales/en-US"),
     "devtools/shared/plural-form": path.join(__dirname, "../../shared/plural-form"),
     "devtools/shared/l10n": path.join(__dirname, "../../shared/l10n"),
     "devtools/shared/system": path.join(projectPath, "system-stub"),
@@ -120,7 +126,7 @@ webpackConfig.resolve = {
     "devtools/shared/fronts/timeline": path.join(__dirname, "../../shared/shims/fronts/timeline"),
     "devtools/shared/defer": path.join(__dirname, "../../shared/defer"),
     "devtools/shared/old-event-emitter": "devtools-modules/src/utils/event-emitter",
-    "devtools/shared/client/main": path.join(__dirname, "new-console-output/test/fixtures/ObjectClient"),
+    "devtools/shared/client/debugger-client": path.join(__dirname, "new-console-output/test/fixtures/DebuggerClient"),
     "devtools/shared/platform/clipboard": path.join(__dirname, "../../shared/platform/content/clipboard"),
     "devtools/shared/platform/stack": path.join(__dirname, "../../shared/platform/content/stack"),
 
