@@ -401,9 +401,11 @@ nsIContent::GetBaseURI(bool aTryUseXHRDocBaseURI) const
     }
 
     // Otherwise check for xml:base attribute
-    elem->GetAttr(kNameSpaceID_XML, nsGkAtoms::base, attr);
-    if (!attr.IsEmpty()) {
-      baseAttrs.AppendElement(attr);
+    if (elem->IsElement()) {
+      elem->AsElement()->GetAttr(kNameSpaceID_XML, nsGkAtoms::base, attr);
+      if (!attr.IsEmpty()) {
+        baseAttrs.AppendElement(attr);
+      }
     }
     elem = elem->GetParent();
   } while(elem);
@@ -1076,43 +1078,6 @@ nsIContent::GetEventTargetParent(EventChainPreVisitor& aVisitor)
   return NS_OK;
 }
 
-bool
-nsIContent::GetAttr(int32_t aNameSpaceID, nsIAtom* aName,
-                    nsAString& aResult) const
-{
-  if (IsElement()) {
-    return AsElement()->GetAttr(aNameSpaceID, aName, aResult);
-  }
-  aResult.Truncate();
-  return false;
-}
-
-bool
-nsIContent::HasAttr(int32_t aNameSpaceID, nsIAtom* aName) const
-{
-  return IsElement() && AsElement()->HasAttr(aNameSpaceID, aName);
-}
-
-bool
-nsIContent::AttrValueIs(int32_t aNameSpaceID,
-                        nsIAtom* aName,
-                        const nsAString& aValue,
-                        nsCaseTreatment aCaseSensitive) const
-{
-  return IsElement() &&
-    AsElement()->AttrValueIs(aNameSpaceID, aName, aValue, aCaseSensitive);
-}
-
-bool
-nsIContent::AttrValueIs(int32_t aNameSpaceID,
-                        nsIAtom* aName,
-                        nsIAtom* aValue,
-                        nsCaseTreatment aCaseSensitive) const
-{
-  return IsElement() &&
-    AsElement()->AttrValueIs(aNameSpaceID, aName, aValue, aCaseSensitive);
-}
-
 bool
 nsIContent::IsFocusable(int32_t* aTabIndex, bool aWithMouse)
 {
@@ -1158,31 +1123,22 @@ FragmentOrElement::DoGetXBLBinding() const
 void
 nsIContent::SetAssignedSlot(HTMLSlotElement* aSlot)
 {
+  MOZ_ASSERT(aSlot || GetExistingExtendedContentSlots());
   ExtendedContentSlots()->mAssignedSlot = aSlot;
 }
 
 void
 nsIContent::SetXBLInsertionPoint(nsIContent* aContent)
 {
-  nsCOMPtr<nsIContent> oldInsertionPoint = nullptr;
   if (aContent) {
     nsExtendedContentSlots* slots = ExtendedContentSlots();
     SetFlags(NODE_MAY_BE_IN_BINDING_MNGR);
-    oldInsertionPoint = slots->mXBLInsertionPoint.forget();
     slots->mXBLInsertionPoint = aContent;
   } else {
     if (nsExtendedContentSlots* slots = GetExistingExtendedContentSlots()) {
-      oldInsertionPoint = slots->mXBLInsertionPoint.forget();
       slots->mXBLInsertionPoint = nullptr;
     }
   }
-
-  // We just changed the flattened tree, so any Servo style data is now invalid.
-  // We rely on nsXBLService::LoadBindings to re-traverse the subtree afterwards.
-  if (oldInsertionPoint != aContent &&
-      IsStyledByServo() && IsElement() && AsElement()->HasServoData()) {
-    ServoRestyleManager::ClearServoDataFromSubtree(AsElement());
-  }
 }
 
 nsresult
@@ -1226,14 +1182,18 @@ FragmentOrElement::SetTextContentInternal(const nsAString& aTextContent,
 void
 FragmentOrElement::DestroyContent()
 {
+  nsIDocument* document = OwnerDoc();
+
   // Drop any servo data. We do this before the RemovedFromDocument call below
   // so that it doesn't need to try to keep the style state sane when shuffling
   // around the flattened tree.
-  if (IsElement() && AsElement()->HasServoData()) {
+  //
+  // TODO(emilio): I suspect this can be asserted against instead, with a bit of
+  // effort to avoid calling nsDocument::Destroy with a shell...
+  if (IsElement() && document->IsStyledByServo()) {
     AsElement()->ClearServoData();
   }
 
-  nsIDocument *document = OwnerDoc();
   document->BindingManager()->RemovedFromDocument(this, document,
                                                   nsBindingManager::eRunDtor);
   document->ClearBoxObjectFor(this);
@@ -2415,40 +2375,3 @@ FragmentOrElement::AddSizeOfExcludingThis(nsWindowSizes& aSizes,
     *aNodeSize += slots->SizeOfIncludingThis(aSizes.mState.mMallocSizeOf);
   }
 }
-
-void
-FragmentOrElement::SetIsElementInStyleScopeFlagOnSubtree(bool aInStyleScope)
-{
-  if (aInStyleScope && IsElementInStyleScope()) {
-    return;
-  }
-
-  if (IsElement()) {
-    SetIsElementInStyleScope(aInStyleScope);
-    SetIsElementInStyleScopeFlagOnShadowTree(aInStyleScope);
-  }
-
-  nsIContent* n = GetNextNode(this);
-  while (n) {
-    if (n->IsElementInStyleScope()) {
-      n = n->GetNextNonChildNode(this);
-    } else {
-      if (n->IsElement()) {
-        n->SetIsElementInStyleScope(aInStyleScope);
-        n->AsElement()->SetIsElementInStyleScopeFlagOnShadowTree(aInStyleScope);
-      }
-      n = n->GetNextNode(this);
-    }
-  }
-}
-
-void
-FragmentOrElement::SetIsElementInStyleScopeFlagOnShadowTree(bool aInStyleScope)
-{
-  NS_ASSERTION(IsElement(), "calling SetIsElementInStyleScopeFlagOnShadowTree "
-                            "on a non-Element is useless");
-  ShadowRoot* shadowRoot = GetShadowRoot();
-  if (shadowRoot) {
-    shadowRoot->SetIsElementInStyleScopeFlagOnSubtree(aInStyleScope);
-  }
-}
