@@ -20,6 +20,12 @@
 class nsCycleCollectionTraversalCallback;
 class nsIPrincipal;
 
+namespace mozilla {
+namespace dom {
+class VoidFunction;
+}  // namespace dom
+}  // namespace mozilla
+
 class nsIGlobalObject : public nsISupports,
                         public mozilla::dom::DispatcherTrait
 {
@@ -76,6 +82,8 @@ public:
 
   virtual bool IsInSyncOperation() { return false; }
 
+  void QueueMicrotask(mozilla::dom::VoidFunction& aCallback);
+
 protected:
   virtual ~nsIGlobalObject();
 
