@@ -1047,9 +1047,9 @@ nsObjectLoadingContent::CheckProcessPolicy(int16_t *aContentPolicy)
 nsObjectLoadingContent::ParameterUpdateFlags
 nsObjectLoadingContent::UpdateObjectParameters()
 {
-  nsCOMPtr<nsIContent> thisContent =
+  nsCOMPtr<Element> thisElement =
     do_QueryInterface(static_cast<nsIImageLoadingContent*>(this));
-  NS_ASSERTION(thisContent, "Must be an instance of content");
+  MOZ_ASSERT(thisElement, "Must be an Element");
 
   uint32_t caps = GetCapabilities();
   LOG(("OBJLC [%p]: Updating object parameters", this));
@@ -1081,7 +1081,7 @@ nsObjectLoadingContent::UpdateObjectParameters()
 
   if (caps & eFallbackIfClassIDPresent) {
     nsAutoString classIDAttr;
-    thisContent->GetAttr(kNameSpaceID_None, nsGkAtoms::classid, classIDAttr);
+    thisElement->GetAttr(kNameSpaceID_None, nsGkAtoms::classid, classIDAttr);
     // We don't support class ID plugin references, so we should always treat
     // having class Ids as attributes as invalid, and fallback accordingly.
     if (!classIDAttr.IsEmpty()) {
@@ -1095,16 +1095,13 @@ nsObjectLoadingContent::UpdateObjectParameters()
   ///
 
   nsAutoString codebaseStr;
-  nsCOMPtr<nsIURI> docBaseURI = thisContent->GetBaseURI();
-  bool hasCodebase = thisContent->HasAttr(kNameSpaceID_None, nsGkAtoms::codebase);
-  if (hasCodebase) {
-    thisContent->GetAttr(kNameSpaceID_None, nsGkAtoms::codebase, codebaseStr);
-  }
+  nsCOMPtr<nsIURI> docBaseURI = thisElement->GetBaseURI();
+  thisElement->GetAttr(kNameSpaceID_None, nsGkAtoms::codebase, codebaseStr);
 
   if (!codebaseStr.IsEmpty()) {
     rv = nsContentUtils::NewURIWithDocumentCharset(getter_AddRefs(newBaseURI),
                                                    codebaseStr,
-                                                   thisContent->OwnerDoc(),
+                                                   thisElement->OwnerDoc(),
                                                    docBaseURI);
     if (NS_SUCCEEDED(rv)) {
       NS_TryToSetImmutable(newBaseURI);
@@ -1116,7 +1113,7 @@ nsObjectLoadingContent::UpdateObjectParameters()
   }
 
   nsAutoString rawTypeAttr;
-  thisContent->GetAttr(kNameSpaceID_None, nsGkAtoms::type, rawTypeAttr);
+  thisElement->GetAttr(kNameSpaceID_None, nsGkAtoms::type, rawTypeAttr);
   if (!rawTypeAttr.IsEmpty()) {
     typeAttr = rawTypeAttr;
     nsAutoString params;
@@ -1136,10 +1133,10 @@ nsObjectLoadingContent::UpdateObjectParameters()
 
   nsAutoString uriStr;
   // Different elements keep this in various locations
-  if (thisContent->NodeInfo()->Equals(nsGkAtoms::object)) {
-    thisContent->GetAttr(kNameSpaceID_None, nsGkAtoms::data, uriStr);
-  } else if (thisContent->NodeInfo()->Equals(nsGkAtoms::embed)) {
-    thisContent->GetAttr(kNameSpaceID_None, nsGkAtoms::src, uriStr);
+  if (thisElement->NodeInfo()->Equals(nsGkAtoms::object)) {
+    thisElement->GetAttr(kNameSpaceID_None, nsGkAtoms::data, uriStr);
+  } else if (thisElement->NodeInfo()->Equals(nsGkAtoms::embed)) {
+    thisElement->GetAttr(kNameSpaceID_None, nsGkAtoms::src, uriStr);
   } else {
     NS_NOTREACHED("Unrecognized plugin-loading tag");
   }
@@ -1150,7 +1147,7 @@ nsObjectLoadingContent::UpdateObjectParameters()
   if (!uriStr.IsEmpty()) {
     rv = nsContentUtils::NewURIWithDocumentCharset(getter_AddRefs(newURI),
                                                    uriStr,
-                                                   thisContent->OwnerDoc(),
+                                                   thisElement->OwnerDoc(),
                                                    newBaseURI);
     nsCOMPtr<nsIURI> rewrittenURI;
     MaybeRewriteYoutubeEmbed(newURI,
@@ -1244,7 +1241,7 @@ nsObjectLoadingContent::UpdateObjectParameters()
     // 5) Use the channel type
 
     bool overrideChannelType = false;
-    if (thisContent->HasAttr(kNameSpaceID_None, nsGkAtoms::typemustmatch)) {
+    if (thisElement->HasAttr(kNameSpaceID_None, nsGkAtoms::typemustmatch)) {
       if (!typeAttr.LowerCaseEqualsASCII(channelType.get())) {
         stateInvalid = true;
       }
@@ -2286,4 +2283,4 @@ nsObjectLoadingContent::BlockEmbedOrObjectContentLoading()
     }
   }
   return false;
-}
\ No newline at end of file
+}
