@@ -1527,7 +1527,7 @@ HTMLFormElement::DoResolveName(const nsAString& aName,
 }
 
 void
-HTMLFormElement::OnSubmitClickBegin(nsIContent* aOriginatingElement)
+HTMLFormElement::OnSubmitClickBegin(Element* aOriginatingElement)
 {
   mDeferSubmission = true;
 
@@ -1595,7 +1595,7 @@ HTMLFormElement::GetAction(nsString& aValue)
 
 nsresult
 HTMLFormElement::GetActionURL(nsIURI** aActionURL,
-                              nsIContent* aOriginatingElement)
+                              Element* aOriginatingElement)
 {
   nsresult rv = NS_OK;
 
@@ -2244,9 +2244,9 @@ HTMLFormElement::WalkRadioGroup(const nsAString& aName,
     for (uint32_t i = 0; i < len; i++) {
       control = GetElementAt(i);
       if (control->ControlType() == NS_FORM_INPUT_RADIO) {
-        nsCOMPtr<nsIContent> controlContent = do_QueryInterface(control);
-        if (controlContent &&
-            controlContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::name,
+        nsCOMPtr<Element> controlElement = do_QueryInterface(control);
+        if (controlElement &&
+            controlElement->AttrValueIs(kNameSpaceID_None, nsGkAtoms::name,
                                         EmptyString(), eCaseMatters) &&
             !aVisitor->Visit(control)) {
           break;
@@ -2443,7 +2443,8 @@ HTMLFormElement::AddElementToTableInternal(
 
       // If an element has a @form, we can assume it *might* be able to not have
       // a parent and still be in the form.
-      NS_ASSERTION(content->HasAttr(kNameSpaceID_None, nsGkAtoms::form) ||
+      NS_ASSERTION((content->IsElement() &&
+                    content->AsElement()->HasAttr(kNameSpaceID_None, nsGkAtoms::form)) ||
                    content->GetParent(), "Item in list without parent");
 
       // Determine the ordering between the new and old element.
