@@ -97,7 +97,7 @@ public:
                NotNull<const Encoding*> aEncoding,
                int32_t aMethod,
                nsIDocument* aDocument,
-               nsIContent* aOriginatingElement)
+               Element* aOriginatingElement)
     : EncodingFormSubmission(aActionURL, aTarget, aEncoding, aOriginatingElement)
     , mMethod(aMethod)
     , mDocument(aDocument)
@@ -401,7 +401,7 @@ FSURLEncoded::URLEncode(const nsAString& aStr, nsACString& aEncoded)
 FSMultipartFormData::FSMultipartFormData(nsIURI* aActionURL,
                                          const nsAString& aTarget,
                                          NotNull<const Encoding*> aEncoding,
-                                         nsIContent* aOriginatingElement)
+                                         Element* aOriginatingElement)
   : EncodingFormSubmission(aActionURL, aTarget, aEncoding, aOriginatingElement)
 {
   mPostData =
@@ -677,7 +677,7 @@ public:
   FSTextPlain(nsIURI* aActionURL,
               const nsAString& aTarget,
               NotNull<const Encoding*> aEncoding,
-              nsIContent* aOriginatingElement)
+              Element* aOriginatingElement)
     : EncodingFormSubmission(aActionURL, aTarget, aEncoding, aOriginatingElement)
   {
   }
@@ -805,7 +805,7 @@ EncodingFormSubmission::EncodingFormSubmission(
   nsIURI* aActionURL,
   const nsAString& aTarget,
   NotNull<const Encoding*> aEncoding,
-  nsIContent* aOriginatingElement)
+  Element* aOriginatingElement)
   : HTMLFormSubmission(aActionURL, aTarget, aEncoding, aOriginatingElement)
 {
   if (!aEncoding->CanEncodeEverything()) {
