@@ -728,6 +728,9 @@ private:
                     bool aSetupOffMainThreadCompositing,
                     bool aSendRegisteredChrome);
 
+  // Generate a minidump for the child process and one for the main process
+  void GeneratePairedMinidump(const char* aReason);
+
   virtual ~ContentParent();
 
   void Init();
@@ -799,8 +802,6 @@ private:
   // Start the force-kill timer on shutdown.
   void StartForceKillTimer();
 
-  void OnGenerateMinidumpComplete(bool aDumpResult);
-
   // Ensure that the permissions for the giben Permission key are set in the
   // content process.
   //
@@ -1186,8 +1187,6 @@ private:
   // nsFakePluginTag::NOT_JSPLUGIN.
   int32_t mJSPluginID;
 
-  nsCString mKillHardAnnotation;
-
   // After we initiate shutdown, we also start a timer to ensure
   // that even content processes that are 100% blocked (say from
   // SIGSTOP), are still killed eventually.  This task enforces that
@@ -1205,6 +1204,7 @@ private:
   // still pass through.
   bool mIsAlive;
 
+  bool mShuttingDown;
   bool mIsForBrowser;
 
   // These variables track whether we've called Close() and KillHard() on our
