@@ -1,5 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim:set ts=2 sw=2 sts=2 et cindent: */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -25,8 +25,6 @@ using mozilla::dom::CreateECParamsForCurve;
 
 const nsCString U2FSoftTokenManager::mSecretNickname =
   NS_LITERAL_CSTRING("U2F_NSSTOKEN");
-const nsString U2FSoftTokenManager::mVersion =
-  NS_LITERAL_STRING("U2F_V2");
 
 namespace {
 NS_NAMED_LITERAL_CSTRING(kAttestCertSubjectName, "CN=Firefox U2F Soft Token");
@@ -539,13 +537,6 @@ PrivateKeyFromKeyHandle(const UniquePK11SlotInfo& aSlot,
   return unwrappedKey;
 }
 
-// Return whether the provided version is supported by this token.
-bool
-U2FSoftTokenManager::IsCompatibleVersion(const nsAString& aVersion)
-{
-  return mVersion == aVersion;
-}
-
 // IsRegistered determines if the provided key handle is usable by this token.
 nsresult
 U2FSoftTokenManager::IsRegistered(const nsTArray<uint8_t>& aKeyHandle,
@@ -592,10 +583,7 @@ U2FSoftTokenManager::IsRegistered(const nsTArray<uint8_t>& aKeyHandle,
 // *      attestation signature
 //
 RefPtr<U2FRegisterPromise>
-U2FSoftTokenManager::Register(const nsTArray<WebAuthnScopedCredentialDescriptor>& aDescriptors,
-                              const nsTArray<uint8_t>& aApplication,
-                              const nsTArray<uint8_t>& aChallenge,
-                              uint32_t aTimeoutMS)
+U2FSoftTokenManager::Register(const WebAuthnMakeCredentialInfo& aInfo)
 {
   if (!mInitialized) {
     nsresult rv = Init();
@@ -604,10 +592,20 @@ U2FSoftTokenManager::Register(const nsTArray<WebAuthnScopedCredentialDescriptor>
     }
   }
 
+  const WebAuthnAuthenticatorSelection& sel = aInfo.AuthenticatorSelection();
+
+  // The U2F softtoken neither supports resident keys or
+  // user verification, nor is it a platform authenticator.
+  if (sel.requireResidentKey() ||
+      sel.requireUserVerification() ||
+      sel.requirePlatformAttachment()) {
+    return U2FRegisterPromise::CreateAndReject(NS_ERROR_DOM_NOT_ALLOWED_ERR, __func__);
+  }
+
   // Optional exclusion list.
-  for (auto desc: aDescriptors) {
+  for (const WebAuthnScopedCredential& cred: aInfo.ExcludeList()) {
     bool isRegistered = false;
-    nsresult rv = IsRegistered(desc.id(), aApplication, isRegistered);
+    nsresult rv = IsRegistered(cred.id(), aInfo.RpIdHash(), isRegistered);
     if (NS_FAILED(rv)) {
       return U2FRegisterPromise::CreateAndReject(rv, __func__);
     }
@@ -641,17 +639,18 @@ U2FSoftTokenManager::Register(const nsTArray<WebAuthnScopedCredentialDescriptor>
   }
 
   // The key handle will be the result of keywrap(privKey, key=mWrappingKey)
-  UniqueSECItem keyHandleItem = KeyHandleFromPrivateKey(slot, mWrappingKey,
-                                                        const_cast<uint8_t*>(aApplication.Elements()),
-                                                        aApplication.Length(),
-                                                        privKey);
+  UniqueSECItem keyHandleItem =
+    KeyHandleFromPrivateKey(slot, mWrappingKey,
+                            const_cast<uint8_t*>(aInfo.RpIdHash().Elements()),
+                            aInfo.RpIdHash().Length(), privKey);
   if (NS_WARN_IF(!keyHandleItem.get())) {
     return U2FRegisterPromise::CreateAndReject(NS_ERROR_FAILURE, __func__);
   }
 
   // Sign the challenge using the Attestation privkey (from attestCert)
   mozilla::dom::CryptoBuffer signedDataBuf;
-  if (NS_WARN_IF(!signedDataBuf.SetCapacity(1 + aApplication.Length() + aChallenge.Length() +
+  if (NS_WARN_IF(!signedDataBuf.SetCapacity(1 + aInfo.RpIdHash().Length() +
+                                            aInfo.ClientDataHash().Length() +
                                             keyHandleItem->len + kPublicKeyLen,
                                             mozilla::fallible))) {
     return U2FRegisterPromise::CreateAndReject(NS_ERROR_OUT_OF_MEMORY, __func__);
@@ -660,8 +659,8 @@ U2FSoftTokenManager::Register(const nsTArray<WebAuthnScopedCredentialDescriptor>
   // // It's OK to ignore the return values here because we're writing into
   // // pre-allocated space
   signedDataBuf.AppendElement(0x00, mozilla::fallible);
-  signedDataBuf.AppendElements(aApplication, mozilla::fallible);
-  signedDataBuf.AppendElements(aChallenge, mozilla::fallible);
+  signedDataBuf.AppendElements(aInfo.RpIdHash(), mozilla::fallible);
+  signedDataBuf.AppendElements(aInfo.ClientDataHash(), mozilla::fallible);
   signedDataBuf.AppendSECItem(keyHandleItem.get());
   signedDataBuf.AppendSECItem(pubKey->u.ec.publicValue);
 
@@ -689,10 +688,31 @@ U2FSoftTokenManager::Register(const nsTArray<WebAuthnScopedCredentialDescriptor>
   registrationBuf.AppendSECItem(attestCert.get()->derCert);
   registrationBuf.AppendSECItem(signatureItem);
 
-  U2FRegisterResult result((nsTArray<uint8_t>(registrationBuf)));
+  WebAuthnMakeCredentialResult result((nsTArray<uint8_t>(registrationBuf)));
   return U2FRegisterPromise::CreateAndResolve(Move(result), __func__);
 }
 
+bool
+U2FSoftTokenManager::FindRegisteredKeyHandle(const nsTArray<nsTArray<uint8_t>>& aAppIds,
+                                             const nsTArray<WebAuthnScopedCredential>& aCredentials,
+                                             /*out*/ nsTArray<uint8_t>& aKeyHandle,
+                                             /*out*/ nsTArray<uint8_t>& aAppId)
+{
+  for (const nsTArray<uint8_t>& app_id: aAppIds) {
+    for (const WebAuthnScopedCredential& cred: aCredentials) {
+      bool isRegistered = false;
+      nsresult rv = IsRegistered(cred.id(), app_id, isRegistered);
+      if (NS_SUCCEEDED(rv) && isRegistered) {
+        aKeyHandle.Assign(cred.id());
+        aAppId.Assign(app_id);
+        return true;
+      }
+    }
+  }
+
+  return false;
+}
+
 // A U2F Sign operation creates a signature over the "param" arguments (plus
 // some other stuff) using the private key indicated in the key handle argument.
 //
@@ -710,23 +730,35 @@ U2FSoftTokenManager::Register(const nsTArray<WebAuthnScopedCredentialDescriptor>
 //  *     Signature
 //
 RefPtr<U2FSignPromise>
-U2FSoftTokenManager::Sign(const nsTArray<WebAuthnScopedCredentialDescriptor>& aDescriptors,
-                          const nsTArray<uint8_t>& aApplication,
-                          const nsTArray<uint8_t>& aChallenge,
-                          uint32_t aTimeoutMS)
+U2FSoftTokenManager::Sign(const WebAuthnGetAssertionInfo& aInfo)
 {
-  nsTArray<uint8_t> keyHandle;
-  for (auto desc: aDescriptors) {
-    bool isRegistered = false;
-    nsresult rv = IsRegistered(desc.id(), aApplication, isRegistered);
-    if (NS_SUCCEEDED(rv) && isRegistered) {
-      keyHandle.Assign(desc.id());
-      break;
+  if (!mInitialized) {
+    nsresult rv = Init();
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return U2FSignPromise::CreateAndReject(rv, __func__);
     }
   }
 
-  // Fail if we didn't recognize a key id.
-  if (keyHandle.IsEmpty()) {
+  // The U2F softtoken doesn't support user verification.
+  if (aInfo.RequireUserVerification()) {
+    return U2FSignPromise::CreateAndReject(NS_ERROR_DOM_NOT_ALLOWED_ERR, __func__);
+  }
+
+  nsTArray<nsTArray<uint8_t>> appIds;
+  appIds.AppendElement(aInfo.RpIdHash());
+
+  // Process extensions.
+  for (const WebAuthnExtension& ext: aInfo.Extensions()) {
+    if (ext.type() == WebAuthnExtension::TWebAuthnExtensionAppId) {
+      appIds.AppendElement(ext.get_WebAuthnExtensionAppId().AppId());
+    }
+  }
+
+  nsTArray<uint8_t> chosenAppId;
+  nsTArray<uint8_t> keyHandle;
+
+  // Fail if we can't find a valid key handle.
+  if (!FindRegisteredKeyHandle(appIds, aInfo.AllowList(), keyHandle, chosenAppId)) {
     return U2FSignPromise::CreateAndReject(NS_ERROR_DOM_NOT_ALLOWED_ERR, __func__);
   }
 
@@ -735,20 +767,23 @@ U2FSoftTokenManager::Sign(const nsTArray<WebAuthnScopedCredentialDescriptor>& aD
   UniquePK11SlotInfo slot(PK11_GetInternalSlot());
   MOZ_ASSERT(slot.get());
 
-  if (NS_WARN_IF((aChallenge.Length() != kParamLen) || (aApplication.Length() != kParamLen))) {
+  if (NS_WARN_IF((aInfo.ClientDataHash().Length() != kParamLen) ||
+                 (chosenAppId.Length() != kParamLen))) {
     MOZ_LOG(gNSSTokenLog, LogLevel::Warning,
             ("Parameter lengths are wrong! challenge=%d app=%d expected=%d",
-             (uint32_t)aChallenge.Length(), (uint32_t)aApplication.Length(), kParamLen));
+             (uint32_t)aInfo.ClientDataHash().Length(),
+             (uint32_t)chosenAppId.Length(), kParamLen));
 
     return U2FSignPromise::CreateAndReject(NS_ERROR_ILLEGAL_VALUE, __func__);
   }
 
   // Decode the key handle
-  UniqueSECKEYPrivateKey privKey = PrivateKeyFromKeyHandle(slot, mWrappingKey,
-                                                           const_cast<uint8_t*>(keyHandle.Elements()),
-                                                           keyHandle.Length(),
-                                                           const_cast<uint8_t*>(aApplication.Elements()),
-                                                           aApplication.Length());
+  UniqueSECKEYPrivateKey privKey =
+    PrivateKeyFromKeyHandle(slot, mWrappingKey,
+                            const_cast<uint8_t*>(keyHandle.Elements()),
+                            keyHandle.Length(),
+                            const_cast<uint8_t*>(chosenAppId.Elements()),
+                            chosenAppId.Length());
   if (NS_WARN_IF(!privKey.get())) {
     MOZ_LOG(gNSSTokenLog, LogLevel::Warning, ("Couldn't get the priv key!"));
     return U2FSignPromise::CreateAndReject(NS_ERROR_FAILURE, __func__);
@@ -777,11 +812,13 @@ U2FSoftTokenManager::Sign(const nsTArray<WebAuthnScopedCredentialDescriptor>& aD
 
   // It's OK to ignore the return values here because we're writing into
   // pre-allocated space
-  signedDataBuf.AppendElements(aApplication.Elements(), aApplication.Length(),
+  signedDataBuf.AppendElements(chosenAppId.Elements(),
+                               chosenAppId.Length(),
                                mozilla::fallible);
   signedDataBuf.AppendElement(0x01, mozilla::fallible);
   signedDataBuf.AppendSECItem(counterItem);
-  signedDataBuf.AppendElements(aChallenge.Elements(), aChallenge.Length(),
+  signedDataBuf.AppendElements(aInfo.ClientDataHash().Elements(),
+                               aInfo.ClientDataHash().Length(),
                                mozilla::fallible);
 
   if (MOZ_LOG_TEST(gNSSTokenLog, LogLevel::Debug)) {
@@ -819,7 +856,15 @@ U2FSoftTokenManager::Sign(const nsTArray<WebAuthnScopedCredentialDescriptor>& aD
   signatureBuf.AppendSECItem(counterItem);
   signatureBuf.AppendSECItem(signatureItem);
 
-  U2FSignResult result(Move(keyHandle), nsTArray<uint8_t>(signatureBuf));
+  nsTArray<uint8_t> signature(signatureBuf);
+  nsTArray<WebAuthnExtensionResult> extensions;
+
+  if (chosenAppId != aInfo.RpIdHash()) {
+    // Indicate to the RP that we used the FIDO appId.
+    extensions.AppendElement(WebAuthnExtensionResultAppId(true));
+  }
+
+  WebAuthnGetAssertionResult result(chosenAppId, keyHandle, signature, extensions);
   return U2FSignPromise::CreateAndResolve(Move(result), __func__);
 }
 
