@@ -227,7 +227,7 @@ ServiceWorkerJob::Finish(ErrorResult& aRv)
 
   // Async release this object to ensure that our caller methods complete
   // as well.
-  NS_ReleaseOnMainThreadSystemGroup("ServiceWorkerJob",
+  NS_ReleaseOnMainThreadSystemGroup("ServiceWorkerJobProxyRunnable",
     kungFuDeathGrip.forget(), true /* always proxy */);
 }
 
