@@ -39,8 +39,6 @@ namespace android {
 class MOZ_EXPORT GraphicBuffer;
 } // namespace android
 
-struct nsStyleFilter;
-
 namespace mozilla {
 namespace layers {
 
@@ -49,6 +47,47 @@ class TextureHost;
 #undef NONE
 #undef OPAQUE
 
+struct LayersId {
+  uint64_t mId;
+
+  bool IsValid() const {
+    return mId != 0;
+  }
+
+  // Allow explicit cast to a uint64_t for now
+  explicit operator uint64_t() const
+  {
+    return mId;
+  }
+
+  // Implement some operators so this class can be used as a key in
+  // stdlib classes.
+  bool operator<(const LayersId& aOther) const
+  {
+    return mId < aOther.mId;
+  }
+
+  bool operator==(const LayersId& aOther) const
+  {
+    return mId == aOther.mId;
+  }
+
+  bool operator!=(const LayersId& aOther) const
+  {
+    return !(*this == aOther);
+  }
+
+  // Helper struct that allow this class to be used as a key in
+  // std::unordered_map like so:
+  //   std::unordered_map<LayersId, ValueType, LayersId::HashFn> myMap;
+  struct HashFn {
+    std::size_t operator()(const LayersId& aKey) const
+    {
+      return std::hash<uint64_t>{}(aKey.mId);
+    }
+  };
+};
+
 enum class LayersBackend : int8_t {
   LAYERS_NONE = 0,
   LAYERS_BASIC,
@@ -102,12 +141,23 @@ struct EventRegions {
   nsIntRegion mHorizontalPanRegion;
   nsIntRegion mVerticalPanRegion;
 
+  // Set to true if events targeting the dispatch-to-content region
+  // require target confirmation.
+  // See CompositorHitTestFlags::eRequiresTargetConfirmation.
+  // We don't bother tracking a separate region for this (which would
+  // be a sub-region of the dispatch-to-content region), because the added
+  // overhead of region computations is not worth it, and because
+  // EventRegions are going to be deprecated anyways.
+  bool mDTCRequiresTargetConfirmation;
+
   EventRegions()
+    : mDTCRequiresTargetConfirmation(false)
   {
   }
 
   explicit EventRegions(nsIntRegion aHitRegion)
     : mHitRegion(aHitRegion)
+    , mDTCRequiresTargetConfirmation(false)
   {
   }
 
@@ -119,7 +169,8 @@ struct EventRegions {
                const nsIntRegion& aDispatchToContentRegion,
                const nsIntRegion& aNoActionRegion,
                const nsIntRegion& aHorizontalPanRegion,
-               const nsIntRegion& aVerticalPanRegion);
+               const nsIntRegion& aVerticalPanRegion,
+               bool aDTCRequiresTargetConfirmation);
 
   bool operator==(const EventRegions& aRegions) const
   {
@@ -127,7 +178,8 @@ struct EventRegions {
            mDispatchToContentHitRegion == aRegions.mDispatchToContentHitRegion &&
            mNoActionRegion == aRegions.mNoActionRegion &&
            mHorizontalPanRegion == aRegions.mHorizontalPanRegion &&
-           mVerticalPanRegion == aRegions.mVerticalPanRegion;
+           mVerticalPanRegion == aRegions.mVerticalPanRegion &&
+           mDTCRequiresTargetConfirmation == aRegions.mDTCRequiresTargetConfirmation;
   }
   bool operator!=(const EventRegions& aRegions) const
   {
@@ -176,6 +228,7 @@ struct EventRegions {
       combinedActionRegions.OrWith(mNoActionRegion);
       mDispatchToContentHitRegion.OrWith(combinedActionRegions);
     }
+    mDTCRequiresTargetConfirmation |= aOther.mDTCRequiresTargetConfirmation;
   }
 
   bool IsEmpty() const
@@ -205,7 +258,7 @@ struct EventRegions {
   }
 };
 
-// Bit flags that go on a ContainerLayer (or RefLayer) and override the
+// Bit flags that go on a RefLayer and override the
 // event regions in the entire subtree below. This is needed for propagating
 // various flags across processes since the child-process layout code doesn't
 // know about parent-process listeners or CSS rules.
@@ -239,6 +292,8 @@ enum TextureDumpMode {
   DoNotCompress  // dump texture uncompressed
 };
 
+typedef uint32_t TouchBehaviorFlags;
+
 // Some specialized typedefs of Matrix4x4Typed.
 typedef gfx::Matrix4x4Typed<LayerPixel, CSSTransformedLayerPixel> CSSTransformMatrix;
 // Several different async transforms can contribute to a layer's transform
@@ -321,57 +376,11 @@ private:
   uint64_t mHandle;
 };
 
-class ReadLockHandle
-{
-  friend struct IPC::ParamTraits<mozilla::layers::ReadLockHandle>;
-public:
-  ReadLockHandle() : mHandle(0)
-  {}
-  ReadLockHandle(const ReadLockHandle& aOther) : mHandle(aOther.mHandle)
-  {}
-  explicit ReadLockHandle(uint64_t aHandle) : mHandle(aHandle)
-  {}
-  bool IsValid() const {
-    return mHandle != 0;
-  }
-  explicit operator bool() const {
-    return IsValid();
-  }
-  bool operator ==(const ReadLockHandle& aOther) const {
-    return mHandle == aOther.mHandle;
-  }
-  uint64_t Value() const {
-    return mHandle;
-  }
-private:
-  uint64_t mHandle;
-};
-
 MOZ_DEFINE_ENUM_CLASS_WITH_BASE(ScrollDirection, uint32_t, (
-  NONE,
-  VERTICAL,
-  HORIZONTAL
+  eVertical,
+  eHorizontal
 ));
 
-enum class CSSFilterType : int8_t {
-  BLUR,
-  BRIGHTNESS,
-  CONTRAST,
-  GRAYSCALE,
-  HUE_ROTATE,
-  INVERT,
-  OPACITY,
-  SATURATE,
-  SEPIA,
-};
-
-struct CSSFilter {
-  CSSFilterType type;
-  float argument;
-};
-
-CSSFilter ToCSSFilter(const nsStyleFilter& filter);
-
 } // namespace layers
 } // namespace mozilla
 
