@@ -45,10 +45,6 @@
 #include "GLBlitTextureImageHelper.h"
 #include "HeapCopyOfStackArray.h"
 
-#if MOZ_WIDGET_ANDROID
-#include "TexturePoolOGL.h"
-#endif
-
 #include "GeckoProfiler.h"
 
 namespace mozilla {
@@ -211,8 +207,6 @@ CompositorOGL::CleanupResources()
 
   mBlitTextureImageHelper = nullptr;
 
-  mContextStateTracker.DestroyOGL(mGLContext);
-
   // On the main thread the Widget will be destroyed soon and calling MakeCurrent
   // after that could cause a crash (at least with GLX, see bug 1059793), unless
   // context is marked as destroyed.
@@ -522,9 +516,9 @@ CompositorOGL::PrepareViewport(CompositingRenderTargetOGL* aRenderTarget)
 already_AddRefed<CompositingRenderTarget>
 CompositorOGL::CreateRenderTarget(const IntRect &aRect, SurfaceInitMode aInit)
 {
-  MOZ_ASSERT(aRect.Width() != 0 && aRect.Height() != 0, "Trying to create a render target of invalid size");
+  MOZ_ASSERT(!aRect.IsZeroArea(), "Trying to create a render target of invalid size");
 
-  if (aRect.Width() * aRect.Height() == 0) {
+  if (aRect.IsZeroArea()) {
     return nullptr;
   }
 
@@ -549,9 +543,9 @@ CompositorOGL::CreateRenderTargetFromSource(const IntRect &aRect,
                                             const CompositingRenderTarget *aSource,
                                             const IntPoint &aSourcePoint)
 {
-  MOZ_ASSERT(aRect.Width() != 0 && aRect.Height() != 0, "Trying to create a render target of invalid size");
+  MOZ_ASSERT(!aRect.IsZeroArea(), "Trying to create a render target of invalid size");
 
-  if (aRect.Width() * aRect.Height() == 0) {
+  if (aRect.IsZeroArea()) {
     return nullptr;
   }
 
@@ -589,10 +583,6 @@ CompositorOGL::SetRenderTarget(CompositingRenderTarget *aSurface)
     = static_cast<CompositingRenderTargetOGL*>(aSurface);
   if (mCurrentRenderTarget != surface) {
     mCurrentRenderTarget = surface;
-    if (mCurrentRenderTarget) {
-      mContextStateTracker.PopOGLSection(gl(), "Frame");
-    }
-    mContextStateTracker.PushOGLSection(gl(), "Frame");
     surface->BindRenderTarget();
   }
 
@@ -620,10 +610,10 @@ void
 CompositorOGL::ClearRect(const gfx::Rect& aRect)
 {
   // Map aRect to OGL coordinates, origin:bottom-left
-  GLint y = mViewportSize.height - (aRect.y + aRect.Height());
+  GLint y = mViewportSize.height - aRect.YMost();
 
   ScopedGLState scopedScissorTestState(mGLContext, LOCAL_GL_SCISSOR_TEST, true);
-  ScopedScissorRect autoScissorRect(mGLContext, aRect.x, y, aRect.Width(), aRect.Height());
+  ScopedScissorRect autoScissorRect(mGLContext, aRect.X(), y, aRect.Width(), aRect.Height());
   mGLContext->fClearColor(0.0, 0.0, 0.0, 0.0);
   mGLContext->fClear(LOCAL_GL_COLOR_BUFFER_BIT | LOCAL_GL_DEPTH_BUFFER_BIT);
 }
@@ -644,15 +634,15 @@ CompositorOGL::BeginFrame(const nsIntRegion& aInvalidRegion,
   if (mUseExternalSurfaceSize) {
     rect = gfx::IntRect(0, 0, mSurfaceSize.width, mSurfaceSize.height);
   } else {
-    rect = gfx::IntRect(aRenderBounds.x, aRenderBounds.y, aRenderBounds.Width(), aRenderBounds.Height());
+    rect = gfx::IntRect(aRenderBounds.X(), aRenderBounds.Y(), aRenderBounds.Width(), aRenderBounds.Height());
   }
 
   if (aRenderBoundsOut) {
     *aRenderBoundsOut = rect;
   }
 
-  GLint width = rect.Width();
-  GLint height = rect.Height();
+  auto width = rect.Width();
+  auto height = rect.Height();
 
   // We can't draw anything to something with no area
   // so just return
@@ -678,10 +668,6 @@ CompositorOGL::BeginFrame(const nsIntRegion& aInvalidRegion,
   mPixelsPerFrame = width * height;
   mPixelsFilled = 0;
 
-#ifdef MOZ_WIDGET_ANDROID
-  TexturePoolOGL::Fill(gl());
-#endif
-
   // Default blend function implements "OVER"
   mGLContext->fBlendFuncSeparate(LOCAL_GL_ONE, LOCAL_GL_ONE_MINUS_SRC_ALPHA,
                                  LOCAL_GL_ONE, LOCAL_GL_ONE_MINUS_SRC_ALPHA);
@@ -731,6 +717,9 @@ CompositorOGL::CreateTexture(const IntRect& aRect, bool aCopyFromSource,
   clampedRect.SetWidth(std::min(clampedRect.Width(), maxTexSize));
   clampedRect.SetHeight(std::min(clampedRect.Height(), maxTexSize));
 
+  auto clampedRectWidth = clampedRect.Width();
+  auto clampedRectHeight = clampedRect.Height();
+
   GLuint tex;
 
   mGLContext->fActiveTexture(LOCAL_GL_TEXTURE0);
@@ -759,25 +748,25 @@ CompositorOGL::CreateTexture(const IntRect& aRect, bool aCopyFromSource,
       mGLContext->fCopyTexImage2D(mFBOTextureTarget,
                                   0,
                                   LOCAL_GL_RGBA,
-                                  clampedRect.x, FlipY(clampedRect.y + clampedRect.Height()),
-                                  clampedRect.Width(), clampedRect.Height(),
+                                  clampedRect.X(), FlipY(clampedRect.YMost()),
+                                  clampedRectWidth, clampedRectHeight,
                                   0);
     } else {
       // Curses, incompatible formats.  Take a slow path.
 
       // RGBA
-      size_t bufferSize = clampedRect.Width() * clampedRect.Height() * 4;
+      size_t bufferSize = clampedRectWidth * clampedRectHeight * 4;
       auto buf = MakeUnique<uint8_t[]>(bufferSize);
 
-      mGLContext->fReadPixels(clampedRect.x, clampedRect.y,
-                              clampedRect.Width(), clampedRect.Height(),
+      mGLContext->fReadPixels(clampedRect.X(), clampedRect.Y(),
+                              clampedRectWidth, clampedRectHeight,
                               LOCAL_GL_RGBA,
                               LOCAL_GL_UNSIGNED_BYTE,
                               buf.get());
       mGLContext->fTexImage2D(mFBOTextureTarget,
                               0,
                               LOCAL_GL_RGBA,
-                              clampedRect.Width(), clampedRect.Height(),
+                              clampedRectWidth, clampedRectHeight,
                               0,
                               LOCAL_GL_RGBA,
                               LOCAL_GL_UNSIGNED_BYTE,
@@ -795,7 +784,7 @@ CompositorOGL::CreateTexture(const IntRect& aRect, bool aCopyFromSource,
     mGLContext->fTexImage2D(mFBOTextureTarget,
                             0,
                             LOCAL_GL_RGBA,
-                            clampedRect.Width(), clampedRect.Height(),
+                            clampedRectWidth, clampedRectHeight,
                             0,
                             LOCAL_GL_RGBA,
                             LOCAL_GL_UNSIGNED_BYTE,
@@ -812,8 +801,8 @@ CompositorOGL::CreateTexture(const IntRect& aRect, bool aCopyFromSource,
   mGLContext->fBindTexture(mFBOTextureTarget, 0);
 
   if (aAllocSize) {
-    aAllocSize->width = clampedRect.Width();
-    aAllocSize->height = clampedRect.Height();
+    aAllocSize->width = clampedRectWidth;
+    aAllocSize->height = clampedRectHeight;
   }
 
   return tex;
@@ -872,7 +861,10 @@ CompositorOGL::GetShaderConfigFor(Effect *aEffect,
     TextureSourceOGL* source = texturedEffect->mTexture->AsSourceOGL();
     MOZ_ASSERT_IF(source->GetTextureTarget() == LOCAL_GL_TEXTURE_EXTERNAL,
                   source->GetFormat() == gfx::SurfaceFormat::R8G8B8A8 ||
-                  source->GetFormat() == gfx::SurfaceFormat::R8G8B8X8);
+                  source->GetFormat() == gfx::SurfaceFormat::R8G8B8X8 ||
+                  source->GetFormat() == gfx::SurfaceFormat::B8G8R8A8 ||
+                  source->GetFormat() == gfx::SurfaceFormat::B8G8R8X8 ||
+                  source->GetFormat() == gfx::SurfaceFormat::R5G6B5_UINT16);
     MOZ_ASSERT_IF(source->GetTextureTarget() == LOCAL_GL_TEXTURE_RECTANGLE_ARB,
                   source->GetFormat() == gfx::SurfaceFormat::R8G8B8A8 ||
                   source->GetFormat() == gfx::SurfaceFormat::R8G8B8X8 ||
@@ -1049,7 +1041,7 @@ CompositorOGL::DrawGeometry(const Geometry& aGeometry,
 
   // XXX: This doesn't handle 3D transforms. It also doesn't handled rotated
   //      quads. Fix me.
-  mPixelsFilled += destRect.width * destRect.height;
+  mPixelsFilled += destRect.Width() * destRect.Height();
 
   // Do a simple culling if this rect is out of target buffer.
   // Inflate a small size to avoid some numerical imprecision issue.
@@ -1062,7 +1054,27 @@ CompositorOGL::DrawGeometry(const Geometry& aGeometry,
 
   LayerScope::DrawBegin();
 
+
   IntRect clipRect = aClipRect;
+
+  EffectMask* effectMask;
+  Rect maskBounds;
+  if (aEffectChain.mSecondaryEffects[EffectTypes::MASK]) {
+    effectMask = static_cast<EffectMask*>(aEffectChain.mSecondaryEffects[EffectTypes::MASK].get());
+
+    // We're assuming that the gl backend won't cheat and use NPOT
+    // textures when glContext says it can't (which seems to happen
+    // on a mac when you force POT textures)
+    IntSize maskSize = CalculatePOTSize(effectMask->mSize, mGLContext);
+
+    const gfx::Matrix4x4& maskTransform = effectMask->mMaskTransform;
+    NS_ASSERTION(maskTransform.Is2D(), "How did we end up with a 3D transform here?!");
+    maskBounds = Rect(Point(), Size(maskSize));
+    maskBounds = maskTransform.As2D().TransformBounds(maskBounds);
+
+    clipRect = clipRect.Intersect(RoundedOut(maskBounds) - offset);
+  }
+
   // aClipRect is in destination coordinate space (after all
   // transforms and offsets have been applied) so if our
   // drawing is going to be shifted by mRenderOffset then we need
@@ -1072,34 +1084,22 @@ CompositorOGL::DrawGeometry(const Geometry& aGeometry,
   }
 
   ScopedGLState scopedScissorTestState(mGLContext, LOCAL_GL_SCISSOR_TEST, true);
-  ScopedScissorRect autoScissorRect(mGLContext, clipRect.x, FlipY(clipRect.y + clipRect.height),
-                                    clipRect.width, clipRect.height);
+  ScopedScissorRect autoScissorRect(mGLContext, clipRect.X(), FlipY(clipRect.Y() + clipRect.Height()),
+                                    clipRect.Width(), clipRect.Height());
 
   MaskType maskType;
-  EffectMask* effectMask;
   TextureSourceOGL* sourceMask = nullptr;
   gfx::Matrix4x4 maskQuadTransform;
   if (aEffectChain.mSecondaryEffects[EffectTypes::MASK]) {
-    effectMask = static_cast<EffectMask*>(aEffectChain.mSecondaryEffects[EffectTypes::MASK].get());
     sourceMask = effectMask->mMaskTexture->AsSourceOGL();
 
     // NS_ASSERTION(textureMask->IsAlpha(),
     //              "OpenGL mask layers must be backed by alpha surfaces");
 
-    // We're assuming that the gl backend won't cheat and use NPOT
-    // textures when glContext says it can't (which seems to happen
-    // on a mac when you force POT textures)
-    IntSize maskSize = CalculatePOTSize(effectMask->mSize, mGLContext);
-
-    const gfx::Matrix4x4& maskTransform = effectMask->mMaskTransform;
-    NS_ASSERTION(maskTransform.Is2D(), "How did we end up with a 3D transform here?!");
-    Rect bounds = Rect(Point(), Size(maskSize));
-    bounds = maskTransform.As2D().TransformBounds(bounds);
-
-    maskQuadTransform._11 = 1.0f/bounds.width;
-    maskQuadTransform._22 = 1.0f/bounds.height;
-    maskQuadTransform._41 = float(-bounds.x)/bounds.width;
-    maskQuadTransform._42 = float(-bounds.y)/bounds.height;
+    maskQuadTransform._11 = 1.0f/maskBounds.Width();
+    maskQuadTransform._22 = 1.0f/maskBounds.Height();
+    maskQuadTransform._41 = float(-maskBounds.X())/maskBounds.Width();
+    maskQuadTransform._42 = float(-maskBounds.Y())/maskBounds.Height();
 
     maskType = MaskType::Mask;
   } else {
@@ -1484,7 +1484,7 @@ CompositorOGL::DrawGeometry(const Geometry& aGeometry,
   MakeCurrent();
 
   LayerScope::DrawEnd(mGLContext, aEffectChain,
-                      aRect.width, aRect.height);
+                      aRect.Width(), aRect.Height());
 }
 
 void
@@ -1608,8 +1608,6 @@ CompositorOGL::EndFrame()
   }
 #endif
 
-  mContextStateTracker.PopOGLSection(gl(), "Frame");
-
   mFrameInProgress = false;
 
   if (mTarget) {
@@ -1692,7 +1690,7 @@ CompositorOGL::CopyToTarget(DrawTarget* aTarget, const nsIntPoint& aTopLeft, con
 
   Matrix oldMatrix = aTarget->GetTransform();
   aTarget->SetTransform(glToCairoTransform);
-  Rect floatRect = Rect(rect.x, rect.y, rect.Width(), rect.Height());
+  Rect floatRect = Rect(rect.X(), rect.Y(), width, height);
   aTarget->DrawSurface(source, floatRect, floatRect, DrawSurfaceOptions(), DrawOptions(1.0f, CompositionOp::OP_SOURCE));
   aTarget->SetTransform(oldMatrix);
   aTarget->Flush();
