@@ -106,6 +106,7 @@ DeviceManagerDx::ReleaseD3D11()
 {
   MOZ_ASSERT(!mCompositorDevice);
   MOZ_ASSERT(!mContentDevice);
+  MOZ_ASSERT(!mVRDevice);
   MOZ_ASSERT(!mDecoderDevice);
 
   mD3D11Module.reset();
@@ -166,6 +167,46 @@ DeviceManagerDx::CreateCompositorDevices()
   return true;
 }
 
+bool
+DeviceManagerDx::CreateVRDevice()
+{
+  MOZ_ASSERT(ProcessOwnsCompositor());
+
+  if (mVRDevice) {
+    return true;
+  }
+
+  if (!gfxConfig::IsEnabled(Feature::D3D11_COMPOSITING)) {
+    NS_WARNING("Direct3D11 Compositing required for VR");
+    return false;
+  }
+
+  if (!LoadD3D11()) {
+    return false;
+  }
+
+  RefPtr<IDXGIAdapter1> adapter = GetDXGIAdapter();
+  if (!adapter) {
+    NS_WARNING("Failed to acquire a DXGI adapter for VR");
+    return false;
+  }
+
+  UINT flags = D3D11_CREATE_DEVICE_BGRA_SUPPORT;
+
+  HRESULT hr;
+  if (!CreateDevice(adapter, D3D_DRIVER_TYPE_UNKNOWN, flags, hr, mVRDevice)) {
+    gfxCriticalError() << "Crash during D3D11 device creation for VR";
+    return false;
+  }
+
+  if (FAILED(hr) || !mVRDevice) {
+    NS_WARNING("Failed to acquire a D3D11 device for VR");
+    return false;
+  }
+
+  return true;
+}
+
 void
 DeviceManagerDx::ImportDeviceInfo(const D3D11DeviceStatus& aDeviceStatus)
 {
@@ -686,7 +727,7 @@ DisableAdvancedLayers(FeatureStatus aStatus, const nsCString aMessage, const nsC
 
   FeatureFailure info(aStatus, aMessage, aFailureId);
   if (GPUParent* gpu = GPUParent::GetSingleton()) {
-    gpu->SendUpdateFeature(Feature::ADVANCED_LAYERS, info);
+    Unused << gpu->SendUpdateFeature(Feature::ADVANCED_LAYERS, info);
   }
 
   if (aFailureId.Length()) {
@@ -742,7 +783,9 @@ DeviceManagerDx::ResetDevices()
   // old and new objects together.
   if (PaintThread::Get()) {
     CompositorBridgeChild* cbc = CompositorBridgeChild::Get();
-    cbc->FlushAsyncPaints();
+    if (cbc) {
+      cbc->FlushAsyncPaints();
+    }
   }
 
   MutexAutoLock lock(mDeviceLock);
@@ -769,10 +812,8 @@ DeviceManagerDx::MaybeResetAndReacquireDevices()
     Telemetry::Accumulate(Telemetry::DEVICE_RESET_REASON, uint32_t(resetReason));
   }
 
-  nsPrintfCString reasonString("%d", int(resetReason));
   CrashReporter::AnnotateCrashReport(
-    NS_LITERAL_CSTRING("DeviceResetReason"),
-    reasonString);
+    CrashReporter::Annotation::DeviceResetReason, int(resetReason));
 
   bool createCompositorDevice = !!mCompositorDevice;
   bool createContentDevice = !!mContentDevice;
@@ -938,6 +979,16 @@ DeviceManagerDx::GetContentDevice()
   return mContentDevice;
 }
 
+RefPtr<ID3D11Device>
+DeviceManagerDx::GetVRDevice()
+{
+  MutexAutoLock lock(mDeviceLock);
+  if (!mVRDevice) {
+    CreateVRDevice();
+  }
+  return mVRDevice;
+}
+
 unsigned
 DeviceManagerDx::GetCompositorFeatureLevel() const
 {
