@@ -2,6 +2,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+extern crate euclid;
 extern crate gleam;
 extern crate glutin;
 extern crate webrender;
@@ -10,6 +11,7 @@ extern crate webrender;
 mod boilerplate;
 
 use boilerplate::{Example, HandyDandyRectBuilder};
+use euclid::SideOffsets2D;
 use webrender::api::*;
 
 struct App {
@@ -22,13 +24,16 @@ impl Example for App {
         _api: &RenderApi,
         builder: &mut DisplayListBuilder,
         _resources: &mut ResourceUpdates,
-        layout_size: LayoutSize,
+        _framebuffer_size: DeviceUintSize,
         _pipeline_id: PipelineId,
         _document_id: DocumentId,
     ) {
-        let info = LayoutPrimitiveInfo::new(LayoutRect::new(LayoutPoint::zero(), layout_size));
+        let info = LayoutPrimitiveInfo::new(
+            LayoutRect::new(LayoutPoint::zero(), builder.content_size())
+        );
         builder.push_stacking_context(
             &info,
+            None,
             ScrollPolicy::Scrollable,
             None,
             TransformStyle::Flat,
@@ -41,9 +46,9 @@ impl Example for App {
             // scrolling and clips stuff
             // let's make a scrollbox
             let scrollbox = (0, 0).to(300, 400);
-            let info = LayoutPrimitiveInfo::new((10, 10).by(0, 0));
             builder.push_stacking_context(
-                &info,
+                &LayoutPrimitiveInfo::new((10, 10).by(0, 0)),
+                None,
                 ScrollPolicy::Scrollable,
                 None,
                 TransformStyle::Flat,
@@ -64,17 +69,20 @@ impl Example for App {
 
             // now put some content into it.
             // start with a white background
-            let info = LayoutPrimitiveInfo::new((0, 0).to(1000, 1000));
+            let mut info = LayoutPrimitiveInfo::new((0, 0).to(1000, 1000));
+            info.tag = Some((0, 1));
             builder.push_rect(&info, ColorF::new(1.0, 1.0, 1.0, 1.0));
 
             // let's make a 50x50 blue square as a visual reference
-            let info = LayoutPrimitiveInfo::new((0, 0).to(50, 50));
+            let mut info = LayoutPrimitiveInfo::new((0, 0).to(50, 50));
+            info.tag = Some((0, 2));
             builder.push_rect(&info, ColorF::new(0.0, 0.0, 1.0, 1.0));
 
             // and a 50x50 green square next to it with an offset clip
             // to see what that looks like
-            let info =
+            let mut info =
                 LayoutPrimitiveInfo::with_clip_rect((50, 0).to(100, 50), (60, 10).to(110, 60));
+            info.tag = Some((0, 3));
             builder.push_rect(&info, ColorF::new(0.0, 1.0, 0.0, 1.0));
 
             // Below the above rectangles, set up a nested scrollbox. It's still in
@@ -92,12 +100,14 @@ impl Example for App {
 
             // give it a giant gray background just to distinguish it and to easily
             // visually identify the nested scrollbox
-            let info = LayoutPrimitiveInfo::new((-1000, -1000).to(5000, 5000));
+            let mut info = LayoutPrimitiveInfo::new((-1000, -1000).to(5000, 5000));
+            info.tag = Some((0, 4));
             builder.push_rect(&info, ColorF::new(0.5, 0.5, 0.5, 1.0));
 
             // add a teal square to visualize the scrolling/clipping behaviour
             // as you scroll the nested scrollbox
-            let info = LayoutPrimitiveInfo::new((0, 200).to(50, 250));
+            let mut info = LayoutPrimitiveInfo::new((0, 200).to(50, 250));
+            info.tag = Some((0, 5));
             builder.push_rect(&info, ColorF::new(0.0, 1.0, 1.0, 1.0));
 
             // Add a sticky frame. It will "stick" twice while scrolling, once
@@ -105,29 +115,23 @@ impl Example for App {
             // and once at a margin of 10px from the top, for 60 pixels of
             // scrolling.
             let sticky_id = builder.define_sticky_frame(
-                None,
                 (50, 350).by(50, 50),
-                StickyFrameInfo::new(
-                    Some(StickySideConstraint {
-                        margin: 10.0,
-                        max_offset: 60.0,
-                    }),
-                    None,
-                    Some(StickySideConstraint {
-                        margin: 10.0,
-                        max_offset: -40.0,
-                    }),
-                    None,
-                ),
+                SideOffsets2D::new(Some(10.0), None, Some(10.0), None),
+                StickyOffsetBounds::new(-40.0, 60.0),
+                StickyOffsetBounds::new(0.0, 0.0),
+                LayoutVector2D::new(0.0, 0.0)
             );
+
             builder.push_clip_id(sticky_id);
-            let info = LayoutPrimitiveInfo::new((50, 350).by(50, 50));
+            let mut info = LayoutPrimitiveInfo::new((50, 350).by(50, 50));
+            info.tag = Some((0, 6));
             builder.push_rect(&info, ColorF::new(0.5, 0.5, 1.0, 1.0));
             builder.pop_clip_id(); // sticky_id
 
             // just for good measure add another teal square further down and to
             // the right, which can be scrolled into view by the user
-            let info = LayoutPrimitiveInfo::new((250, 350).to(300, 400));
+            let mut info = LayoutPrimitiveInfo::new((250, 350).to(300, 400));
+            info.tag = Some((0, 7));
             builder.push_rect(&info, ColorF::new(0.0, 1.0, 1.0, 1.0));
 
             builder.pop_clip_id(); // nested_clip_id
@@ -139,9 +143,17 @@ impl Example for App {
         builder.pop_stacking_context();
     }
 
-    fn on_event(&mut self, event: glutin::Event, api: &RenderApi, document_id: DocumentId) -> bool {
+    fn on_event(&mut self, event: glutin::WindowEvent, api: &RenderApi, document_id: DocumentId) -> bool {
+        let mut txn = Transaction::new();
         match event {
-            glutin::Event::KeyboardInput(glutin::ElementState::Pressed, _, Some(key)) => {
+            glutin::WindowEvent::KeyboardInput {
+                input: glutin::KeyboardInput {
+                    state: glutin::ElementState::Pressed,
+                    virtual_keycode: Some(key),
+                    ..
+                },
+                ..
+            } => {
                 let offset = match key {
                     glutin::VirtualKeyCode::Down => (0.0, -10.0),
                     glutin::VirtualKeyCode::Up => (0.0, 10.0),
@@ -150,37 +162,45 @@ impl Example for App {
                     _ => return false,
                 };
 
-                api.scroll(
-                    document_id,
+                txn.scroll(
                     ScrollLocation::Delta(LayoutVector2D::new(offset.0, offset.1)),
                     self.cursor_position,
-                    ScrollEventPhase::Start,
                 );
             }
-            glutin::Event::MouseMoved(x, y) => {
+            glutin::WindowEvent::CursorMoved { position: (x, y), .. } => {
                 self.cursor_position = WorldPoint::new(x as f32, y as f32);
             }
-            glutin::Event::MouseWheel(delta, _, event_cursor_position) => {
-                if let Some((x, y)) = event_cursor_position {
-                    self.cursor_position = WorldPoint::new(x as f32, y as f32);
-                }
-
+            glutin::WindowEvent::MouseWheel { delta, .. } => {
                 const LINE_HEIGHT: f32 = 38.0;
                 let (dx, dy) = match delta {
                     glutin::MouseScrollDelta::LineDelta(dx, dy) => (dx, dy * LINE_HEIGHT),
                     glutin::MouseScrollDelta::PixelDelta(dx, dy) => (dx, dy),
                 };
 
-                api.scroll(
-                    document_id,
+                txn.scroll(
                     ScrollLocation::Delta(LayoutVector2D::new(dx, dy)),
                     self.cursor_position,
-                    ScrollEventPhase::Start,
                 );
             }
+            glutin::WindowEvent::MouseInput { .. } => {
+                let results = api.hit_test(
+                    document_id,
+                    None,
+                    self.cursor_position,
+                    HitTestFlags::FIND_ALL
+                );
+
+                println!("Hit test results:");
+                for item in &results.items {
+                    println!("  • {:?}", item);
+                }
+                println!("");
+            }
             _ => (),
         }
 
+        api.send_transaction(document_id, txn);
+
         false
     }
 }
