@@ -8,7 +8,6 @@
 #include "nsString.h"
 #include "nsIStringBundle.h"
 #include "nsStringBundleService.h"
-#include "nsStringBundleTextOverride.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIMutableArray.h"
 #include "nsArrayEnumerator.h"
@@ -41,10 +40,8 @@ static NS_DEFINE_CID(kErrorServiceCID, NS_ERRORSERVICE_CID);
 
 NS_IMPL_ISUPPORTS(nsStringBundle, nsIStringBundle)
 
-nsStringBundle::nsStringBundle(const char* aURLSpec,
-                               nsIStringBundleOverride* aOverrideStrings) :
+nsStringBundle::nsStringBundle(const char* aURLSpec) :
   mPropertiesURL(aURLSpec),
-  mOverrideStrings(aOverrideStrings),
   mReentrantMonitor("nsStringBundle.mReentrantMonitor"),
   mAttemptedLoad(false),
   mLoaded(false)
@@ -64,6 +61,26 @@ nsStringBundle::AsyncPreload()
                           &nsStringBundle::LoadProperties));
 }
 
+size_t
+nsStringBundle::SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const
+{
+  size_t n = 0;
+  if (mProps) {
+    n += mProps->SizeOfIncludingThis(aMallocSizeOf);
+  }
+  return aMallocSizeOf(this) + n;
+}
+
+size_t
+nsStringBundle::SizeOfIncludingThisIfUnshared(MallocSizeOf aMallocSizeOf) const
+{
+  if (mRefCnt == 1) {
+    return SizeOfIncludingThis(aMallocSizeOf);
+  } else {
+    return 0;
+  }
+}
+
 nsresult
 nsStringBundle::LoadProperties()
 {
@@ -158,14 +175,6 @@ nsStringBundle::GetStringFromName(const char* aName, nsAString& aResult)
 
   ReentrantMonitorAutoEnter automon(mReentrantMonitor);
 
-  // try override first
-  if (mOverrideStrings) {
-    rv = mOverrideStrings->GetStringFromName(mPropertiesURL,
-                                             nsDependentCString(aName),
-                                             aResult);
-    if (NS_SUCCEEDED(rv)) return rv;
-  }
-
   return mProps->GetStringProperty(nsDependentCString(aName), aResult);
 }
 
@@ -207,71 +216,6 @@ nsStringBundle::FormatStringFromName(const char* aName,
   return FormatString(formatStr.get(), aParams, aLength, aResult);
 }
 
-nsresult
-nsStringBundle::GetCombinedEnumeration(nsIStringBundleOverride* aOverrideStrings,
-                                       nsISimpleEnumerator** aResult)
-{
-  nsCOMPtr<nsISupports> supports;
-  nsCOMPtr<nsIPropertyElement> propElement;
-
-  nsresult rv;
-
-  nsCOMPtr<nsIMutableArray> resultArray =
-    do_CreateInstance(NS_ARRAY_CONTRACTID, &rv);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // first, append the override elements
-  nsCOMPtr<nsISimpleEnumerator> overrideEnumerator;
-  rv = aOverrideStrings->EnumerateKeysInBundle(mPropertiesURL,
-                                               getter_AddRefs(overrideEnumerator));
-
-  bool hasMore;
-  rv = overrideEnumerator->HasMoreElements(&hasMore);
-  NS_ENSURE_SUCCESS(rv, rv);
-  while (hasMore) {
-
-    rv = overrideEnumerator->GetNext(getter_AddRefs(supports));
-    if (NS_SUCCEEDED(rv))
-      resultArray->AppendElement(supports);
-
-    rv = overrideEnumerator->HasMoreElements(&hasMore);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  // ok, now we have the override elements in resultArray
-  nsCOMPtr<nsISimpleEnumerator> propEnumerator;
-  rv = mProps->Enumerate(getter_AddRefs(propEnumerator));
-  if (NS_FAILED(rv)) {
-    // no elements in mProps anyway, just return what we have
-    return NS_NewArrayEnumerator(aResult, resultArray);
-  }
-
-  // second, append all the elements that are in mProps
-  do {
-    rv = propEnumerator->GetNext(getter_AddRefs(supports));
-    if (NS_SUCCEEDED(rv) &&
-        (propElement = do_QueryInterface(supports, &rv))) {
-
-      // now check if its in the override bundle
-      nsAutoCString key;
-      propElement->GetKey(key);
-
-      nsAutoString value;
-      rv = aOverrideStrings->GetStringFromName(mPropertiesURL, key, value);
-
-      // if it isn't there, then it is safe to append
-      if (NS_FAILED(rv))
-        resultArray->AppendElement(propElement);
-    }
-
-    rv = propEnumerator->HasMoreElements(&hasMore);
-    NS_ENSURE_SUCCESS(rv, rv);
-  } while (hasMore);
-
-  return resultArray->Enumerate(aResult);
-}
-
-
 NS_IMETHODIMP
 nsStringBundle::GetSimpleEnumeration(nsISimpleEnumerator** elements)
 {
@@ -282,9 +226,6 @@ nsStringBundle::GetSimpleEnumeration(nsISimpleEnumerator** elements)
   rv = LoadProperties();
   if (NS_FAILED(rv)) return rv;
 
-  if (mOverrideStrings)
-      return GetCombinedEnumeration(mOverrideStrings, elements);
-
   return mProps->Enumerate(elements);
 }
 
@@ -316,150 +257,6 @@ nsStringBundle::FormatString(const char16_t *aFormatStr,
   return NS_OK;
 }
 
-NS_IMPL_ISUPPORTS(nsExtensibleStringBundle, nsIStringBundle)
-
-nsExtensibleStringBundle::nsExtensibleStringBundle()
-{
-  mLoaded = false;
-}
-
-nsresult
-nsExtensibleStringBundle::Init(const char * aCategory,
-                               nsIStringBundleService* aBundleService)
-{
-
-  nsresult rv;
-  nsCOMPtr<nsICategoryManager> catman =
-    do_GetService(NS_CATEGORYMANAGER_CONTRACTID, &rv);
-  if (NS_FAILED(rv)) return rv;
-
-  nsCOMPtr<nsISimpleEnumerator> enumerator;
-  rv = catman->EnumerateCategory(aCategory, getter_AddRefs(enumerator));
-  if (NS_FAILED(rv)) return rv;
-
-  bool hasMore;
-  while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore) {
-    nsCOMPtr<nsISupports> supports;
-    rv = enumerator->GetNext(getter_AddRefs(supports));
-    if (NS_FAILED(rv))
-      continue;
-
-    nsCOMPtr<nsISupportsCString> supStr = do_QueryInterface(supports, &rv);
-    if (NS_FAILED(rv))
-      continue;
-
-    nsAutoCString name;
-    rv = supStr->GetData(name);
-    if (NS_FAILED(rv))
-      continue;
-
-    nsCOMPtr<nsIStringBundle> bundle;
-    rv = aBundleService->CreateBundle(name.get(), getter_AddRefs(bundle));
-    if (NS_FAILED(rv))
-      continue;
-
-    mBundles.AppendObject(bundle);
-  }
-
-  return rv;
-}
-
-NS_IMETHODIMP
-nsExtensibleStringBundle::AsyncPreload()
-{
-  nsresult rv = NS_OK;
-  const uint32_t size = mBundles.Count();
-  for (uint32_t i = 0; i < size; ++i) {
-    nsIStringBundle* bundle = mBundles[i];
-    if (bundle) {
-      nsresult rv2 = bundle->AsyncPreload();
-      rv = NS_FAILED(rv) ? rv : rv2;
-    }
-  }
-  return rv;
-}
-
-nsExtensibleStringBundle::~nsExtensibleStringBundle()
-{
-}
-
-nsresult
-nsExtensibleStringBundle::GetStringFromID(int32_t aID, nsAString& aResult)
-{
-  nsAutoCString idStr;
-  idStr.AppendInt(aID, 10);
-  return GetStringFromName(idStr.get(), aResult);
-}
-
-nsresult
-nsExtensibleStringBundle::GetStringFromAUTF8Name(const nsACString& aName,
-                                                 nsAString& aResult)
-{
-  return GetStringFromName(PromiseFlatCString(aName).get(), aResult);
-}
-
-nsresult
-nsExtensibleStringBundle::GetStringFromName(const char* aName,
-                                            nsAString& aResult)
-{
-  nsresult rv;
-  const uint32_t size = mBundles.Count();
-  for (uint32_t i = 0; i < size; ++i) {
-    nsIStringBundle* bundle = mBundles[i];
-    if (bundle) {
-      rv = bundle->GetStringFromName(aName, aResult);
-      if (NS_SUCCEEDED(rv))
-        return NS_OK;
-    }
-  }
-
-  return NS_ERROR_FAILURE;
-}
-
-NS_IMETHODIMP
-nsExtensibleStringBundle::FormatStringFromID(int32_t aID,
-                                             const char16_t ** aParams,
-                                             uint32_t aLength,
-                                             nsAString& aResult)
-{
-  nsAutoCString idStr;
-  idStr.AppendInt(aID, 10);
-  return FormatStringFromName(idStr.get(), aParams, aLength, aResult);
-}
-
-NS_IMETHODIMP
-nsExtensibleStringBundle::FormatStringFromAUTF8Name(const nsACString& aName,
-                                                    const char16_t ** aParams,
-                                                    uint32_t aLength,
-                                                    nsAString& aResult)
-{
-  return FormatStringFromName(PromiseFlatCString(aName).get(),
-                              aParams, aLength, aResult);
-}
-
-NS_IMETHODIMP
-nsExtensibleStringBundle::FormatStringFromName(const char* aName,
-                                               const char16_t** aParams,
-                                               uint32_t aLength,
-                                               nsAString& aResult)
-{
-  nsAutoString formatStr;
-  nsresult rv;
-  rv = GetStringFromName(aName, formatStr);
-  if (NS_FAILED(rv))
-    return rv;
-
-  return nsStringBundle::FormatString(formatStr.get(), aParams, aLength,
-                                      aResult);
-}
-
-nsresult nsExtensibleStringBundle::GetSimpleEnumeration(nsISimpleEnumerator ** aResult)
-{
-  // XXX write me
-  *aResult = nullptr;
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 /////////////////////////////////////////////////////////////////////////////////////////
 
 #define MAX_CACHED_BUNDLES 16
@@ -490,10 +287,12 @@ nsStringBundleService::nsStringBundleService() :
 NS_IMPL_ISUPPORTS(nsStringBundleService,
                   nsIStringBundleService,
                   nsIObserver,
-                  nsISupportsWeakReference)
+                  nsISupportsWeakReference,
+                  nsIMemoryReporter)
 
 nsStringBundleService::~nsStringBundleService()
 {
+  UnregisterWeakMemoryReporter(this);
   flushBundleCache();
 }
 
@@ -505,18 +304,25 @@ nsStringBundleService::Init()
     os->AddObserver(this, "memory-pressure", true);
     os->AddObserver(this, "profile-do-change", true);
     os->AddObserver(this, "chrome-flush-caches", true);
-    os->AddObserver(this, "xpcom-category-entry-added", true);
     os->AddObserver(this, "intl:app-locales-changed", true);
   }
 
-  // instantiate the override service, if there is any.
-  // at some point we probably want to make this a category, and
-  // support multiple overrides
-  mOverrideStrings = do_GetService(NS_STRINGBUNDLETEXTOVERRIDE_CONTRACTID);
+  RegisterWeakMemoryReporter(this);
 
   return NS_OK;
 }
 
+size_t
+nsStringBundleService::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
+{
+  size_t n = mBundleMap.ShallowSizeOfExcludingThis(aMallocSizeOf);
+  for (auto iter = mBundleMap.ConstIter(); !iter.Done(); iter.Next()) {
+    n += iter.Data()->mBundle->SizeOfIncludingThis(aMallocSizeOf);
+    n += iter.Data()->mHashKey.SizeOfExcludingThisIfUnshared(aMallocSizeOf);
+  }
+  return aMallocSizeOf(this) + n;
+}
+
 NS_IMETHODIMP
 nsStringBundleService::Observe(nsISupports* aSubject,
                                const char* aTopic,
@@ -529,11 +335,6 @@ nsStringBundleService::Observe(nsISupports* aSubject,
   {
     flushBundleCache();
   }
-  else if (strcmp("xpcom-category-entry-added", aTopic) == 0 &&
-           NS_LITERAL_STRING("xpcom-autoregistration").Equals(aSomeData))
-  {
-    mOverrideStrings = do_GetService(NS_STRINGBUNDLETEXTOVERRIDE_CONTRACTID);
-  }
 
   return NS_OK;
 }
@@ -570,9 +371,8 @@ nsStringBundleService::getStringBundle(const char *aURLSpec,
     cacheEntry->remove();
 
   } else {
-
     // hasn't been cached, so insert it into the hash table
-    RefPtr<nsStringBundle> bundle = new nsStringBundle(aURLSpec, mOverrideStrings);
+    RefPtr<nsStringBundle> bundle = new nsStringBundle(aURLSpec);
     cacheEntry = insertIntoCache(bundle.forget(), key);
   }
 
@@ -626,24 +426,6 @@ nsStringBundleService::CreateBundle(const char* aURLSpec,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsStringBundleService::CreateExtensibleBundle(const char* aCategory,
-                                              nsIStringBundle** aResult)
-{
-  NS_ENSURE_ARG_POINTER(aResult);
-  *aResult = nullptr;
-
-  RefPtr<nsExtensibleStringBundle> bundle = new nsExtensibleStringBundle();
-
-  nsresult res = bundle->Init(aCategory, this);
-  if (NS_FAILED(res)) {
-    return res;
-  }
-
-  bundle.forget(aResult);
-  return NS_OK;
-}
-
 #define GLOBAL_PROPERTIES "chrome://global/locale/global-strres.properties"
 
 nsresult
