@@ -18,6 +18,7 @@
 #include "IPCMessageStart.h"
 #include "mozilla/AlreadyAddRefed.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/ipc/ByteBuf.h"
 #include "mozilla/ipc/FileDescriptor.h"
 #include "mozilla/ipc/Shmem.h"
 #include "mozilla/ipc/Transport.h"
@@ -29,6 +30,7 @@
 #include "mozilla/NotNull.h"
 #include "mozilla/UniquePtr.h"
 #include "MainThreadUtils.h"
+#include "nsICrashReporter.h"
 #include "nsILabelableRunnable.h"
 
 #if defined(ANDROID) && defined(DEBUG)
@@ -280,11 +282,9 @@ public:
 
     ProtocolId GetProtocolId() const { return mProtocolId; }
 
-    base::ProcessId OtherPid() const;
+    base::ProcessId OtherPid() const override;
     void SetOtherProcessId(base::ProcessId aOtherPid);
 
-    bool TakeMinidump(nsIFile** aDump, uint32_t* aSequence);
-
     virtual void OnChannelClose() = 0;
     virtual void OnChannelError() = 0;
     virtual void ProcessingError(Result aError, const char* aMsgName) {}
@@ -307,16 +307,16 @@ public:
 
     void SetReplyTimeoutMs(int32_t aTimeoutMs);
 
-    virtual int32_t Register(IProtocol*);
-    virtual int32_t RegisterID(IProtocol*, int32_t);
-    virtual IProtocol* Lookup(int32_t);
-    virtual void Unregister(int32_t);
+    virtual int32_t Register(IProtocol*) override;
+    virtual int32_t RegisterID(IProtocol*, int32_t) override;
+    virtual IProtocol* Lookup(int32_t) override;
+    virtual void Unregister(int32_t) override;
 
     virtual Shmem::SharedMemory* CreateSharedMemory(
-        size_t, SharedMemory::SharedMemoryType, bool, int32_t*);
-    virtual Shmem::SharedMemory* LookupSharedMemory(int32_t);
-    virtual bool IsTrackingSharedMemory(Shmem::SharedMemory*);
-    virtual bool DestroySharedMemory(Shmem&);
+        size_t, SharedMemory::SharedMemoryType, bool, int32_t*) override;
+    virtual Shmem::SharedMemory* LookupSharedMemory(int32_t) override;
+    virtual bool IsTrackingSharedMemory(Shmem::SharedMemory*) override;
+    virtual bool DestroySharedMemory(Shmem&) override;
 
     void DeallocShmems();
 
@@ -403,7 +403,7 @@ public:
     GetActorEventTarget(IProtocol* aActor);
 
     virtual nsIEventTarget*
-    GetActorEventTarget();
+    GetActorEventTarget() override;
 
     virtual void OnChannelReceivedMessage(const Message& aMsg) {}
 
@@ -421,13 +421,14 @@ protected:
     virtual already_AddRefed<nsIEventTarget>
     GetSpecificMessageEventTarget(const Message& aMsg) { return nullptr; }
 
-    virtual void SetEventTargetForActorInternal(IProtocol* aActor, nsIEventTarget* aEventTarget);
+    virtual void SetEventTargetForActorInternal(IProtocol* aActor,
+                                                nsIEventTarget* aEventTarget) override;
     virtual void ReplaceEventTargetForActorInternal(
       IProtocol* aActor,
-      nsIEventTarget* aEventTarget);
+      nsIEventTarget* aEventTarget) override;
 
     virtual already_AddRefed<nsIEventTarget>
-    GetActorEventTargetInternal(IProtocol* aActor);
+    GetActorEventTargetInternal(IProtocol* aActor) override;
 
   private:
     ProtocolId mProtocolId;
@@ -622,18 +623,22 @@ public:
 
     Endpoint(Endpoint&& aOther)
       : mValid(aOther.mValid)
-      , mMode(aOther.mMode)
       , mTransport(aOther.mTransport)
       , mMyPid(aOther.mMyPid)
       , mOtherPid(aOther.mOtherPid)
     {
+        if (aOther.mValid) {
+            mMode = aOther.mMode;
+        }
         aOther.mValid = false;
     }
 
     Endpoint& operator=(Endpoint&& aOther)
     {
         mValid = aOther.mValid;
-        mMode = aOther.mMode;
+        if (aOther.mValid) {
+            mMode = aOther.mMode;
+        }
         mTransport = aOther.mTransport;
         mMyPid = aOther.mMyPid;
         mOtherPid = aOther.mOtherPid;
@@ -689,9 +694,9 @@ private:
 };
 
 #if defined(XP_MACOSX)
-void AnnotateCrashReportWithErrno(const char* tag, int error);
+void AnnotateCrashReportWithErrno(CrashReporter::Annotation tag, int error);
 #else
-static inline void AnnotateCrashReportWithErrno(const char* tag, int error)
+static inline void AnnotateCrashReportWithErrno(CrashReporter::Annotation tag, int error)
 {}
 #endif
 
@@ -711,7 +716,8 @@ CreateEndpoints(const PrivateIPDLInterface& aPrivate,
   TransportDescriptor parentTransport, childTransport;
   nsresult rv;
   if (NS_FAILED(rv = CreateTransport(aParentDestPid, &parentTransport, &childTransport))) {
-    AnnotateCrashReportWithErrno("IpcCreateEndpointsNsresult", int(rv));
+    AnnotateCrashReportWithErrno(
+      CrashReporter::Annotation::IpcCreateEndpointsNsresult, int(rv));
     return rv;
   }
 
@@ -728,8 +734,6 @@ void
 TableToArray(const nsTHashtable<nsPtrHashKey<void>>& aTable,
              nsTArray<void*>& aArray);
 
-const char* StringFromIPCMessageType(uint32_t aMessageType);
-
 } // namespace ipc
 
 template<typename Protocol>
